#!/usr/bin/env python3
import pickle,os,json,math,shutil
import numpy as np
import pandas as pd
import tkinter as tk
from tkinter import ttk
from plateypus.setup.createTubeLayout import TubeLayoutPage 
from plateypus.setup.createPlateLayout import BlankSelectionPage
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'

experimentParameters = {}
dataTypeDict = {'Cytokine':'cyt','Bulk/Single cell':'cell','Killing':'killing','PCR':'pcr'}

class ExperimentSetupStartPage(tk.Frame):
    def __init__(self, master,fName,bPage):
        global folderName,backPage
        folderName = fName
        backPage = bPage
        tk.Frame.__init__(self, master)
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        v2 = tk.StringVar(value='inpt')
        rb2a = tk.Radiobutton(mainWindow, text="Enter experiment labels ",padx = 20, variable=v2, value='inpt')
        rb2b = tk.Radiobutton(mainWindow,text="Create experiment layout ",padx = 20, variable=v2, value='pl')
        rb2a.grid(row=0,column=0,sticky=tk.W)
        rb2b.grid(row=1,column=0,sticky=tk.W)
        
        #v3 = tk.StringVar(value='both')
        #l3 = tk.Label(mainWindow, text="for:     ")
        #l3.grid(row=0,column=1,rowspan=2,sticky=tk.EW)
        #rb3a = tk.Radiobutton(mainWindow, text="Cells",padx = 20, variable=v3, value='cell')
        #rb3b = tk.Radiobutton(mainWindow,text="Cytokines",padx = 20, variable=v3, value='cyt')
        #rb3c = tk.Radiobutton(mainWindow,text="Cells and Cytokines",padx = 20, variable=v3, value='both')
        #rb3a.grid(row=0,column=2,sticky=tk.W)
        #rb3b.grid(row=1,column=2,sticky=tk.W)
        #rb3c.grid(row=2,column=2,sticky=tk.W)
        
        labelList,cbList,cbVarList = [],[],[]
        for i,dataT in enumerate(list(dataTypeDict.keys())):
            l = tk.Label(mainWindow,text=dataT).grid(row=i,column=3,sticky=tk.W)
            v = tk.BooleanVar(value=False)
            cb = tk.Checkbutton(mainWindow, variable=v)
            cb.grid(row=i,column=1,sticky=tk.W)
            cbList.append(cb)
            cbVarList.append(v)
        
        def experimentLayout():
            for dataType in dataTypeList:
                if 'experimentParameters-'+folderName+'-'+dataType+'.json' in os.listdir('misc'):
                    experimentParameters = json.load(open('misc'+dirSep+'experimentParameters-'+folderName+'-'+dataType+'.json','r'))
                    break
            if 'format' not in experimentParameters.keys():
                experimentParameters['format'] = 'plate'
            if experimentParameters['format'] == 'plate':
                plateDimensions = experimentParameters['overallPlateDimensions']
                levelLabelDict = experimentParameters['levelLabelDict']
                conditionLevelValues = levelLabelDict.copy()
                del levelLabelDict.copy()[list(levelLabelDict.keys())[-1]]
                levelValues = []
                for level in experimentParameters['levelLabelDict']:
                    levelValues.append(experimentParameters['levelLabelDict'][level])
                maxNumLevelValues = len(max(levelValues,key=len))
                levels = list(experimentParameters['levelLabelDict'].keys())
                if 'killing' in dataTypeList:
                    levels = levels[:-1]
                    levelValues = levelValues[:-1]
                master.switch_frame(BlankSelectionPage,folderName,levels,levelValues,maxNumLevelValues,experimentParameters['numPlates'],plateDimensions,dataTypeList,ExperimentSetupStartPage,bPage)
            #Tube mode
            else:
                fcsFiles = []
                
                if 'A1_cell.csv' not in os.listdir('inputData'+dirSep+'bulkCSVFiles'+dirSep):
                    if 'sampleNameFile.xlsx' in os.listdir('misc'):
                        print('waaaaaaaaat2')
                        fcsFiles = list(pd.read_excel('misc'+dirSep+'sampleNameFile.xlsx',header=0).iloc[:,0].values.ravel())
                    elif len(os.listdir('inputData'+dirSep+'fcsFiles')) > 1:
                        for fcsName in os.listdir('inputData'+dirSep+'fcsFiles'):
                            if '.fcs' in fcsName:
                                fcsFiles.append(fcsName)
                        if len(fcsFiles) == 0:
                            fcsFiles = ['                 ']
                        print(os.listdir('inputData'+dirSep+'fcsFiles'))
                else:
                    bulkStatFile = pd.read_csv('inputData'+dirSep+'bulkCSVFiles'+dirSep+'A1_cell.csv')
                    for row in range(bulkStatFile.shape[0]):
                        if bulkStatFile.iloc[row,0] not in ['Mean','SD']:
                            fcsFiles.append(bulkStatFile.iloc[row,0])

                master.switch_frame(TubeLayoutPage,folderName,experimentParameters['levelLabelDict'],len(fcsFiles),'cell',ExperimentSetupStartPage,bPage)
        
        def collectInput():
            global dataTypeList
            dataTypeList = [list(dataTypeDict.values())[i] for i,x in enumerate(cbVarList) if x.get()]
            if v2.get() == 'inpt':
                master.switch_frame(ExperimentFormatPage,folderName)
            elif v2.get() == 'pl':
                experimentLayout()

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInput()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class ExperimentFormatPage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        v1 = tk.StringVar()
        l1 = tk.Label(mainWindow,text='Collection Format: ')
        rb2a = tk.Radiobutton(mainWindow, text="Tube",padx = 20, variable=v1, value='tube')
        rb2b = tk.Radiobutton(mainWindow,text="Plate",padx = 20, variable=v1, value='plate')
        v1.set('tube')
        
        l1.grid(row=0,column=0)
        rb2a.grid(row=1,column=0)
        rb2b.grid(row=2,column=0)
        
        def collectInputs():
            if v1.get() == 'tube':
                experimentParameters['format'] = 'tube'
                master.switch_frame(TubeExperimentParameterPage,folderName)
            else:
                experimentParameters['format'] = 'plate'
                master.switch_frame(PlateExperimentParameterPage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ExperimentSetupStartPage,folderName,backPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class TubeExperimentParameterPage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        

        l4 = tk.Label(mainWindow, text="How many samples are there in this experiment?")
        e0 = tk.Entry(mainWindow)
        l4.grid(row=0,column=0)
        e0.grid(row=0,column=1)
        
        l5 = tk.Label(mainWindow, text="How many unique levels are there in this experiment (excluding Time)?")
        e1 = tk.Entry(mainWindow)
        l5.grid(row=2,column=0)
        e1.grid(row=2,column=1)
        
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc'+dirSep+'sampleNameFile.xlsx')
            columns = []
            for column in df:
                if column not in ['','FileName','Time']:
                    columns.append(column)
            e0.insert(tk.END,str(df.shape[0]))
            e1.insert(tk.END,str(len(columns)))

        def collectInputs():
            experimentParameters['numSamples'] = int(e0.get())
            experimentParameters['numAllLevels'] = int(e1.get())+1
            master.switch_frame(allLevelNamePage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ExperimentFormatPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class PlateExperimentParameterPage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        
        Multiplexing_Options = {'96->384 Well Plate','Barcoding'}

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l7 = tk.Label(mainWindow, text="Multiplexing Options:")
        multiplex = tk.StringVar(value='None')
        plateMultiplexVar = tk.BooleanVar() 
        plateMultiplexVar2 = tk.BooleanVar() 
        barcodeMultiplexVar = tk.BooleanVar() 
        plateMultiplexCb = tk.Checkbutton(mainWindow,text='96->384 well',variable=plateMultiplexVar,onvalue=True,offvalue=False)
        plateMultiplexCbRev = tk.Checkbutton(mainWindow,text='384->96 well',variable=plateMultiplexVar2,onvalue=True,offvalue=False)
        plateMultiplexCbRev.config(state=tk.DISABLED)
        plateMultiplexCb.config(state=tk.NORMAL)
        barcodeMultiplexCb = tk.Checkbutton(mainWindow,text='Barcoding',variable=barcodeMultiplexVar,onvalue=True,offvalue=False)
        barcodingNumberLabel = tk.Label(mainWindow,text='# plates per barcoded plate') 
        barcodingNumberEntry = tk.Entry(mainWindow,width=4)

        v = tk.IntVar()
        v2 = tk.IntVar()
        v3 = tk.IntVar()
        v4 = tk.IntVar()
        
        def disable():
            if v3.get() == 384:
                plateMultiplexCb.config(state=tk.DISABLED)
                plateMultiplexCbRev.config(state=tk.NORMAL)
            else:
                plateMultiplexCbRev.config(state=tk.DISABLED)
                plateMultiplexCb.config(state=tk.NORMAL)

        l3 = tk.Label(mainWindow, text="""What format were the samples collected in?:""")
        rb3a = tk.Radiobutton(mainWindow, text="96 well plate",padx = 20, variable=v3, value=96,command=lambda: disable())
        rb3b = tk.Radiobutton(mainWindow,text="384 well plate",padx = 20, variable=v3, value=384,command=lambda: disable())
        l3.grid(row=0,column=0)
        rb3a.grid(row=0,column=1)
        rb3b.grid(row=0,column=2)
        v3.set(96)

        l4 = tk.Label(mainWindow, text="How many plates were used in the experiment?")
        e0 = tk.Entry(mainWindow)
        l4.grid(row=1,column=0)
        e0.grid(row=1,column=1)

        l6 = tk.Label(mainWindow, text="Enter the number of condition levels (excluding Time): ")
        e2 = tk.Entry(mainWindow)
        l6.grid(row=3,column=0)
        e2.grid(row=3,column=1)
        
        l7.grid(row=4,column=0)
        plateMultiplexCb.grid(row=4,column=1)
        plateMultiplexCbRev.grid(row=5,column=1)
        barcodeMultiplexCb.grid(row=4,column=2)
        barcodingNumberLabel.grid(row=4,column=3)
        barcodingNumberEntry.grid(row=4,column=4)
         
        def collectInputs():
            experimentParameters['numPlates'] = int(e0.get())
            
            #Incucyte datasets have time encoded in them already
            if 'killing' in dataTypeList:
                experimentParameters['numAllLevels'] = int(e2.get())
            else:
                experimentParameters['numAllLevels'] = int(e2.get())+1
            if v3.get() == 384:
                experimentParameters['overallPlateDimensions'] = [16,24]
            else:
                experimentParameters['overallPlateDimensions'] = [8,12]
            
            if (plateMultiplexVar.get() or plateMultiplexVar2.get()) and barcodeMultiplexVar.get():
                multiplexingOption = '96->384 well + Barcoding'
                numberOfBarcodes = barcodingNumberEntry.get()
            else:
                if plateMultiplexVar.get() or plateMultiplexVar2.get():
                    multiplexingOption = '96->384 well'
                    numberOfBarcodes = 1
                elif barcodeMultiplexVar.get():
                    multiplexingOption = 'Barcoding'
                    numberOfBarcodes = barcodingNumberEntry.get()
                else:
                    multiplexingOption = 'None'
                    numberOfBarcodes = 1
            if ',' in barcodingNumberEntry.get():
                numberOfBarcodes = list(map(int,numberOfBarcodes.split(',')))
            else:
                numberOfBarcodes = [int(numberOfBarcodes)]
            global globalMultiplexingVar
            globalMultiplexingVar = multiplexingOption
            experimentParameters['multiplexingOption'] = multiplexingOption
            if multiplexingOption == '96->384 well':
                master.switch_frame(MultiplexingPage,multiplexingOption)
            elif multiplexingOption == '96->384 well + Barcoding':
                master.switch_frame(BarcodingPage,multiplexingOption,numberOfBarcodes)
            elif multiplexingOption == 'Barcoding':
                master.switch_frame(BarcodingPage,multiplexingOption,numberOfBarcodes)
            else:
                master.switch_frame(allLevelNamePage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ExperimentFormatPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class BarcodingPage(tk.Frame):
    def __init__(self, master,mpo,nb):
        tk.Frame.__init__(self, master)
        multiplexingOption = mpo
        BarcodingWindow = tk.Frame(self)
        BarcodingWindow.pack()
        numPl = experimentParameters['numPlates'] 
        
        numberOfBarcodes = math.ceil(np.log2(max(nb)))
        entryBarcodeLabelList = []
        for barcode in range(numberOfBarcodes):
            e = tk.Entry(BarcodingWindow,width=8)
            e.grid(row=0,column=2+barcode)
            e.insert(tk.END, 'Barcode '+str(barcode+1))
            entryBarcodeLabelList.append(e)
            #tk.Label(BarcodingWindow,text='Barcode '+str(barcode+1)).grid(row=0,column=2+barcode)
        
        if len(nb) == 1:
            numberOfBarcodedPlates = nb[0]
            if multiplexingOption == '96->384 well + Barcoding':
                if experimentParameters['overallPlateDimensions'][0] == 8: 
                    maxNumBarcoded = math.ceil(experimentParameters['numPlates'] / 4 / numberOfBarcodedPlates)
                    totalPlatesPerBarcodedPlate = 4*numberOfBarcodedPlates
                    plateSkip = 4
                #384->96
                else: 
                    maxNumBarcoded = math.ceil(4*experimentParameters['numPlates'] / numberOfBarcodedPlates)
                    totalPlatesPerBarcodedPlate = numberOfBarcodedPlates
                    plateSkip = 1 
            else:
                maxNumBarcoded = math.ceil(experimentParameters['numPlates'] / numberOfBarcodedPlates)
                totalPlatesPerBarcodedPlate = numberOfBarcodedPlates
                plateSkip = 1
            allIndividualPlateEntryList = []
            combinedPlateBarcodingVarList,combinedEntryList = [],[]
            for combinedPlateNum in range(maxNumBarcoded):
                combinedEntry = tk.Entry(BarcodingWindow,width=8)
                combinedEntry.insert(tk.END, 'A'+str(combinedPlateNum*totalPlatesPerBarcodedPlate+1)+'-'+str((combinedPlateNum+1)*totalPlatesPerBarcodedPlate))
                plateStart = combinedPlateNum*totalPlatesPerBarcodedPlate+1
                plateEnd = (combinedPlateNum+1)*totalPlatesPerBarcodedPlate
                allPlateIndices = list(range(plateStart,plateEnd+2,plateSkip))
                combinedEntry.grid(row=combinedPlateNum*numberOfBarcodedPlates+1,column=0)
                plateBarcodingVarList,plateEntryList = [],[]
                for plateNum in range(numberOfBarcodedPlates):
                    if plateNum == numberOfBarcodedPlates-1:
                        padvar = 10 
                    else:
                        padvar = 0
                    plateEntry = tk.Entry(BarcodingWindow,width=8)
                    if plateSkip == 4:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum])+'-'+str(allPlateIndices[plateNum+1]-1))
                    else:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum]))
                    plateEntry.grid(row=combinedPlateNum*numberOfBarcodedPlates+plateNum+1,column=1,pady=(0,padvar))
                    barcodingCbList,barcodingVarList = [],[]
                    for barcodeNum in range(numberOfBarcodes):
                        barcodeVar = tk.BooleanVar()
                        barcodeCb = tk.Checkbutton(BarcodingWindow,text='',variable=barcodeVar,onvalue=True,offvalue=False)
                        barcodeCb.grid(row=combinedPlateNum*numberOfBarcodedPlates+plateNum+1,column=2+barcodeNum,pady=(0,padvar))
                        barcodingCbList.append(barcodeCb)
                        barcodingVarList.append(barcodeVar)
                    plateBarcodingVarList.append(barcodingVarList)
                    plateEntryList.append(plateEntry)
                combinedPlateBarcodingVarList.append(plateBarcodingVarList)
                combinedEntryList.append(combinedEntry)
                allIndividualPlateEntryList.append(plateEntryList)
        else:
            if multiplexingOption == '96->384 well + Barcoding':
                if experimentParameters['overallPlateDimensions'][0] == 8: 
                    plateSkip = 4
                #384->96
                else: 
                    plateSkip = 1 
            else:
                plateSkip = 1
            allIndividualPlateEntryList = []
            combinedPlateBarcodingVarList,combinedEntryList = [],[]
            for elem,numberOfBarcodedPlates in enumerate(nb):
                summedPlateNums = sum(nb[:elem+1])
                plateEnd = plateSkip*summedPlateNums
                if elem == 0:
                    plateStart = 1
                    plateRow = 1
                else:
                    plateStart = sum(nb[:elem])*plateSkip + 1
                    plateRow = sum(nb[:elem]) + 1
                combinedEntry = tk.Entry(BarcodingWindow,width=8)
                combinedEntry.insert(tk.END, 'A'+str(plateStart)+'-'+str(plateEnd))
                combinedEntry.grid(row=plateRow,column=0)
                allPlateIndices = list(range(plateStart,plateEnd+2,plateSkip))
                plateBarcodingVarList,plateEntryList = [],[]
                for plateNum in range(numberOfBarcodedPlates):
                    if plateNum == numberOfBarcodedPlates-1 and plateNum != 0:
                        padvar = 10 
                    else:
                        padvar = 0
                    plateEntry = tk.Entry(BarcodingWindow,width=8)
                    if plateSkip == 4:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum])+'-'+str(allPlateIndices[plateNum+1]-1))
                    else:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum]))
                    plateEntry.grid(row=plateRow+plateNum,column=1,pady=(0,padvar))
                    barcodingCbList,barcodingVarList = [],[]
                    for barcodeNum in range(numberOfBarcodes):
                        barcodeVar = tk.BooleanVar()
                        barcodeCb = tk.Checkbutton(BarcodingWindow,text='',variable=barcodeVar,onvalue=True,offvalue=False)
                        barcodeCb.grid(row=plateRow+plateNum,column=2+barcodeNum,pady=(0,padvar))
                        barcodingCbList.append(barcodeCb)
                        barcodingVarList.append(barcodeVar)
                    plateBarcodingVarList.append(barcodingVarList)
                    plateEntryList.append(plateEntry)
                combinedPlateBarcodingVarList.append(plateBarcodingVarList)
                combinedEntryList.append(combinedEntry)
                allIndividualPlateEntryList.append(plateEntryList)

        l1 = tk.Label(BarcodingWindow,text='Barcoded Plate Name:')
        l2 = tk.Label(BarcodingWindow,text='Plate Name:')
        l1.grid(row=0,column=0)
        l2.grid(row=0,column=1)

        def collectInputs():
            barcodingDict = {}
            for combinedIndex,combinedEntry in enumerate(combinedEntryList):
                if combinedEntry.get() != '':
                    plateEntryList = allIndividualPlateEntryList[combinedIndex]
                    plateBarcodingDict = {}
                    for plateIndex,plateEntry in enumerate(plateEntryList):
                        if plateEntry.get() != '':
                            barcodingStringList = []
                            for barcodeIndex,barcodeVar in enumerate(combinedPlateBarcodingVarList[combinedIndex][plateIndex]):
                                if barcodeVar.get():
                                    barcodeSymbol = '+'
                                else:
                                    barcodeSymbol = '-'
                                barcodingStringList.append(entryBarcodeLabelList[barcodeIndex].get()+barcodeSymbol)
                            uncombinedPlateName = plateEntry.get()
                            if '_cyt' in plateEntry.get():
                                uncombinedPlateName = uncombinedPlateName[:-4]
                            elif '_cell' in plateEntry.get():
                                uncombinedPlateName = uncombinedPlateName[:-5]
                            plateBarcodingDict[uncombinedPlateName] = barcodingStringList
                    combinedPlateName = combinedEntry.get()
                    if '_cyt' in combinedEntry.get():
                        combinedPlateName = combinedPlateName[:-4]
                    elif '_cell' in combinedEntry.get():
                        combinedPlateName = combinedPlateName[:-5]
                    barcodingDict[combinedPlateName] = plateBarcodingDict
            experimentParameters['barcodingDict'] = barcodingDict
            if multiplexingOption == '96->384 well + Barcoding':
                master.switch_frame(MultiplexingPage,multiplexingOption)
            else:
                master.switch_frame(allLevelNamePage,folderName)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlateExperimentParameterPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class MultiplexingPage(tk.Frame):
    def __init__(self, master,mpo):
        tk.Frame.__init__(self, master)
        multiplexingOption = mpo
        Well384ConversionWindow = tk.Frame(self)
        Well384ConversionWindow.pack()
        
        reverse = False
        if multiplexingOption == '96->384 well + Barcoding':
            if experimentParameters['overallPlateDimensions'][0] == 16: 
                reverse = True

        if not reverse:
            maxNum384Plates = math.ceil(experimentParameters['numPlates'] / 4)
        else:
            maxNum384Plates = math.ceil(experimentParameters['numPlates'])
        Unpacking384PlateNameList = []
        Unpacking384List = []
        l1 = tk.Label(Well384ConversionWindow,text='Combined Plate Names: ')
        l1.grid(row=0,column=0)
        l2 = tk.Label(Well384ConversionWindow,text='Separated Plate Names: ')
        l2.grid(row=1,column=0)
        ttk.Separator().place(x=0, y=27, relwidth=1)
        ttk.Separator().place(x=157, y=0, relheight=0.65)
        for plateNum in range(maxNum384Plates):
            e1 = tk.Entry(Well384ConversionWindow,width=7)
            if not reverse:
                e1.insert(tk.END, 'A'+str(plateNum*4+1)+'-'+str((plateNum+1)*4))
            else:
                e1.insert(tk.END, 'A'+str(plateNum+1))
            e1.grid(row=0,column=plateNum*2+1)
            Unpacking384PlateNameList.append(e1)
            UnpackingWellPosList = []
            wellIndexList = [0,1,3,2]
            wellIndex = 0
            ttk.Separator().place(x=317+plateNum*160, y=0, relheight=0.65)
            pN = 0
            for wellUnpackingRow in range(2):
                for wellUnpackingCol in range(2):
                    e2 = tk.Entry(Well384ConversionWindow,width=7)
                    if pN < experimentParameters['numPlates']:
                        e2.insert(tk.END, 'A'+str((plateNum*4)+wellIndexList[wellIndex]+1))
                        pN+=1
                    else:
                        e2.insert(tk.END, '')
                    if wellUnpackingCol == 1:
                        padvar = 5
                    else:
                        padvar = 0
                    e2.grid(row=wellUnpackingRow+1,column=wellUnpackingCol+1+plateNum*2,padx=padvar)
                    UnpackingWellPosList.append(e2)
                    wellIndex+=1
            Unpacking384List.append(UnpackingWellPosList)

        def collectInputs():
            unpackingDict = {}
            for unpackingPlateNameEntry,unpackingWellPosList in zip(Unpacking384PlateNameList,Unpacking384List):
                tempList = []
                for unpackingEntry in unpackingWellPosList:
                    tempList.append(unpackingEntry.get())
                unpackingDict[unpackingPlateNameEntry.get()] = tempList
            experimentParameters['unpackingDict'] = unpackingDict
            master.switch_frame(allLevelNamePage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlateExperimentParameterPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)
        
class allLevelNamePage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        numAllLevels = experimentParameters['numAllLevels']
            
        entryList1 = []
        entryList2 = []
        
        numericCheckBoxes = []
        numericCheckBoxVars = []

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        lt1 = tk.Label(mainWindow, text="Level Name").grid(row=0,column=1)
        lt2 = tk.Label(mainWindow, text="Number of Level Values").grid(row=0,column=2)
        lt3 = tk.Label(mainWindow, text="Numeric?").grid(row=0,column=3)
        
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc'+dirSep+'sampleNameFile.xlsx')
            levels = []
            levelValueNums = []
            for level in df.columns:
                if level not in ['','FileName']:
                    levels.append(level)
                    levelValueNums.append(len(pd.unique(df[level])))
            j=0
        
        for conditionLevelNumber in range(1,numAllLevels+1):
            l1 = tk.Label(mainWindow, text="Condition "+str(conditionLevelNumber))
            e1 = tk.Entry(mainWindow)
            v = tk.IntVar()
            if conditionLevelNumber == 1 and 'killing' not in dataTypeList:
                #e1 = tk.Entry(mainWindow,state='readonly')
                e1.insert(tk.END, 'Time')
                e2 = tk.Entry(mainWindow)
                if 'sampleNameFile.xlsx' in os.listdir('misc'):
                    if 'Time' in df.columns:
                        e2.insert(tk.END,str(len(pd.unique(df['Time']))))
                    else: 
                        e2.insert(tk.END, '1')
                else:
                    e2.insert(tk.END, '1')
                v.set(1)
            else:
                e1 = tk.Entry(mainWindow)
                e2 = tk.Entry(mainWindow)
                v.set(0)
                if 'sampleNameFile.xlsx' in os.listdir('misc'):
                    e1.insert(tk.END, levels[j])
                    e2.insert(tk.END, str(levelValueNums[j]))
                    j+=1
                                    
            
            cb1 = tk.Checkbutton(mainWindow, text="",variable=v,onvalue=1,offvalue=0)
             
            l1.grid(row=conditionLevelNumber+1,column=0)
            e1.grid(row=conditionLevelNumber+1,column=1)
            e2.grid(row=conditionLevelNumber+1,column=2)
            entryList1.append(e1)
            entryList2.append(e2)
            cb1.grid(row=conditionLevelNumber+1,column=3)
            numericCheckBoxes.append(cb1)
            numericCheckBoxVars.append(v)
        
        def collectInputs():
            conditionNames = []
            numConditionLevelValues = []
            tiledLevels = []
            experimentParameters['columnVariableName'] = 'Time' 
            numericlevels = []
            for allLevelNumber in range(numAllLevels):
                #Remove column variable from condition name list
                if entryList1[allLevelNumber].get() == 'Time':
                    experimentParameters['numColumnLevelValues'] = int(entryList2[allLevelNumber].get())
                else:
                    conditionNames.append(str(entryList1[allLevelNumber].get()))
                    numConditionLevelValues.append(int(entryList2[allLevelNumber].get()))
                numericlevels.append(numericCheckBoxVars[allLevelNumber].get() == 1)
            if 'killing' in dataTypeList:
                experimentParameters['numColumnLevelValues'] = 0 
                experimentParameters['numConditionLevels'] = numAllLevels
            else:
                experimentParameters['numConditionLevels'] = numAllLevels - 1

            experimentParameters['conditionLevelNames'] = conditionNames
            experimentParameters['allLevelNames'] = [experimentParameters['columnVariableName']]+conditionNames
            experimentParameters['numConditionLevelValues'] = numConditionLevelValues
            experimentParameters['numericLevels'] = numericlevels
            experimentParameters[''] = tiledLevels
            master.switch_frame(columnLevelValuesPage,folderName)

        def backCommand():
            if experimentParameters['format'] == 'tube':
                master.switch_frame(TubeExperimentParameterPage,folderName)
            else:
                master.switch_frame(PlateExperimentParameterPage,folderName)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=numAllLevels+1,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: backCommand()).grid(row=numAllLevels+1,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=numAllLevels+1,column=2)

class columnLevelValuesPage(tk.Frame):
    def __init__(self, master,folderName):
        numColumnLevelValues = experimentParameters['numColumnLevelValues']
        
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        lt = tk.Label(mainWindow,text=experimentParameters['columnVariableName']+':').grid(row=0,column=0)
        col_wrap = 12
        for col in range(1,numColumnLevelValues+1):
            lt1 = tk.Label(mainWindow, text='Level Value '+str(col),width=10).grid(row=int((col-1)/col_wrap)*2,column=((col-1)%col_wrap)+1)
        entryList = []
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc'+dirSep+'sampleNameFile.xlsx')
            if 'Time' in df.columns:
                times = list(map(float,list(pd.unique(df['Time']))))
                sortedTimes = list(map(str,sorted(times)))
                j=0

        for columnLevelValueNumber in range(numColumnLevelValues):
            e1 = tk.Entry(mainWindow,width=10)
            e1.grid(row=2*int(columnLevelValueNumber/col_wrap)+1,column=(columnLevelValueNumber%col_wrap)+1)
            if 'sampleNameFile.xlsx' in os.listdir('misc'):
                if 'Time' in df.columns:
                    e1.insert(tk.END,sortedTimes[j]) 
                    j+=1
            entryList.append(e1)

        def collectInputs():
            columnLevelValues = []
            for entry in entryList:
                columnLevelValues.append(float(entry.get()))
            experimentParameters['columnLevelValues'] = columnLevelValues
            master.switch_frame(conditionLevelValuesPage,folderName)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=2*int(numColumnLevelValues/col_wrap)+2,column=5)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(allLevelNamePage,folderName)).grid(row=2*int(numColumnLevelValues/col_wrap)+2,column=6)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=2*int(numColumnLevelValues/col_wrap)+2,column=7)

class LinkedEntryButton(tk.Button):
    def __init__(self,parent,linkedentries,**kwargs):
        tk.Button.__init__(self,parent,**kwargs)
        self.parent = parent
        self.linkedentries = linkedentries

    def fillNumerically(self):
        for i,entry in enumerate(self.linkedentries):
            entry.insert(tk.END,str(i+1))

class conditionLevelValuesPage(tk.Frame):
    def __init__(self, master,folderName):
        numConditionLevels = experimentParameters['numConditionLevels']
        maxLevelValues = max(experimentParameters['numConditionLevelValues'])
        
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        entryWrap = 16
        fullEntryList = []
        blist = []

        labelList = experimentParameters['conditionLevelNames']
        rowNum = 0
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc'+dirSep+'sampleNameFile.xlsx')
        for conditionLevelNumber in range(numConditionLevels):
            if 'sampleNameFile.xlsx' in os.listdir('misc'):
                j = 0
            currentLabel = labelList[conditionLevelNumber]
            initialRowNum = rowNum
            l1 = tk.Label(mainWindow, text='Level values for \"'+currentLabel+'\":').grid(row=rowNum,column=1,sticky=tk.W) 
            levelEntryList = []
            rowNum-=1
            for col in range(1,maxLevelValues+1):
                if 'sampleNameFile.xlsx' in os.listdir('misc'):
                    for column in df.columns:
                        if column == currentLabel:
                            filledInLevelValues = list(pd.unique(df[column]))
                if (col-1)%entryWrap == 0:
                    rowNum+=1
                if col < experimentParameters['numConditionLevelValues'][conditionLevelNumber]+1:
                    e1 = tk.Entry(mainWindow,width=8)
                    e1.grid(row=rowNum,column=(col-1)%entryWrap+2,sticky=tk.W)
                    if 'sampleNameFile.xlsx' in os.listdir('misc'):
                        e1.insert(tk.END,filledInLevelValues[j])
                        j+=1
                    levelEntryList.append(e1)
            b = LinkedEntryButton(mainWindow,levelEntryList,text='Fill Values Numerically')
            b.configure(command=b.fillNumerically)
            b.grid(row=initialRowNum,column=0,sticky=tk.W)
            blist.append(b)
            fullEntryList.append(levelEntryList)
            rowNum+=1

        def collectInputs():
            conditionLevels = {}
            for lvlentrylist,i in zip(fullEntryList,range(numConditionLevels)):
                tempLevels = []
                for entry in lvlentrylist:
                    tempLevels.append(str(entry.get()))
                conditionLevels[experimentParameters['conditionLevelNames'][i]] = tempLevels
            experimentParameters['conditionLevelValues'] = conditionLevels.copy()
            experimentParameters['allLevelValues'] = conditionLevels.copy()
            experimentParameters['allLevelValues'][experimentParameters['columnVariableName']] = experimentParameters['columnLevelValues']
            shortenedExperimentParameters = {}
            for keyToKeep in ['format','numPlates','overallPlateDimensions','barcodingDict','unpackingDict']:
                if keyToKeep in list(experimentParameters.keys()):
                    shortenedExperimentParameters[keyToKeep] = experimentParameters[keyToKeep]
            shortenedExperimentParameters['levelLabelDict'] = experimentParameters['allLevelValues']
            for dataType in dataTypeList:
                with open('misc'+dirSep+'experimentParameters-'+folderName+'-'+dataType+'.json', 'w') as fp:
                    json.dump(shortenedExperimentParameters, fp)
            #{"A1-12": {"A1-4": ["Barcode 1-", "Barcode 2-"], "A5-8": ["Barcode 1+", "Barcode 2-"], "A9-12": ["Barcode 1-", "Barcode 2+"]}}
            #{"A1-4": ["A1", "A2", "A4", "A3"], "A5-8": ["A5", "A6", "A8", "A7"], "A9-12": ["A9", "A10", "A12", "A11"]}
            if 'cell' in dataTypeList and experimentParameters['format'] == 'plate':
                #Both
                if '+' in globalMultiplexingVar:
                    for plate in experimentParameters['barcodingDict']:
                        if plate not in os.listdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep):
                            os.mkdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep+plate)
                else:
                    if 'Barcoding' in globalMultiplexingVar:
                        for plate in experimentParameters['barcodingDict']:
                            if plate not in os.listdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep):
                                os.mkdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep+plate)
                    elif '96' in globalMultiplexingVar:
                        for plate in experimentParameters['unpackingDict']:
                            if plate not in os.listdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep):
                                os.mkdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep+plate)
                    else:
                        for plateNum in range(1,experimentParameters['numPlates']+1):
                            if 'A'+str(plateNum) not in os.listdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep):
                                os.mkdir('inputData'+dirSep+'singleCellCSVFiles'+dirSep+'A'+str(plateNum))
            master.switch_frame(ExperimentSetupStartPage,folderName,backPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=numConditionLevels+1,column=int(maxLevelValues/2))
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(columnLevelValuesPage,folderName)).grid(row=numConditionLevels+1,column=int(maxLevelValues/2)+1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=numConditionLevels+1,column=int(maxLevelValues/2)+2)



#!/usr/bin/env python3 
import json,pickle,math,matplotlib,sys,glob
from sys import platform as sys_pf
if sys_pf == 'darwin':
    import matplotlib
    matplotlib.use("TkAgg")
import colorcet as cc
import numpy as np
import pandas as pd
import seaborn as sns
import tkinter as tk
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit
from itertools import groupby
from tkinter.ttk import Combobox
from tkinter.messagebox import askyesno,showinfo
import os
import warnings,shutil
warnings.filterwarnings("ignore")
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'
from plateypus.dataprocessing.miscFunctions import Hill,InverseHill,r_squared,cleanUpFlowjoCSV, setMaxWidth
idx = pd.IndexSlice

class CytokineParsingPage(tk.Frame):
    def __init__(self, master,completeCytokineMWDf,unparsedCytokines,folderName,expNum,ex_data,shp,bPage):
        tk.Frame.__init__(self, master)
        
        secondaryhomepage = shp
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text='Several cytokine names in raw data were not found in loaded CBA kit.\nPlease choose their correct names below:').pack()
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10) 
        
        tk.Label(mainWindow, text='Unparsed Name:').grid(row=0, column=0)
        tk.Label(mainWindow, text='Kit Name:').grid(row=0, column=1)
        
        kitCytokineNames = list(pd.unique(completeCytokineMWDf.reset_index()['Cytokine']))
        kitCytokineNameVarList = []
        for i,unparsedCytokine in enumerate(unparsedCytokines):
            tk.Label(mainWindow, text=unparsedCytokine).grid(row=i+1, column=0)
            kitCytokineNameVar = tk.StringVar()
            kitCytokineNameDropdown = tk.OptionMenu(mainWindow,kitCytokineNameVar,*kitCytokineNames)
            kitCytokineNameDropdown.grid(row=i+1, column=1)
            kitCytokineNameVarList.append(kitCytokineNameVar)
         
        def collectInputs():
            cytokineRenamingDict = {x:y.get() for x,y in zip(unparsedCytokines,kitCytokineNameVarList)}
            with open('misc'+dirSep+'cytRenamingDict-'+folderName+'.json','w') as f:
                json.dump(cytokineRenamingDict,f)
            master.switch_frame(secondaryhomepage,folderName,expNum,ex_data,bPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class CalibrationParameterPage(tk.Frame):
    def __init__(self, master,folderName,expNum,ex_data,shp,bPage):
        tk.Frame.__init__(self, master)
        
        secondaryhomepage = shp
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10) 
        
        l1 = tk.Label(mainWindow,text='Number of CBA calibration samples: ').grid(row=0,column=0)
        t1 = tk.Entry(mainWindow)
        t1.grid(row=1,column=0,sticky=tk.W)

        l2 = tk.Label(mainWindow,text='Volume of initial CBA calibration solution (mL): ').grid(row=0,column=1)
        t2 = tk.Entry(mainWindow)
        t2.grid(row=1,column=1,sticky=tk.W)
        
        l4 = tk.Label(mainWindow,text='Serial Dilution Factor: ').grid(row=0,column=2)
        t4 = tk.Entry(mainWindow)
        t4.grid(row=1,column=2,sticky=tk.W)
        
        l3 = tk.Label(mainWindow, text='Species of CBA samples: ').grid(row=0, column=3)

        kitDf = pd.read_csv(master.homedirectory+'misc'+dirSep+'kitDf.csv').set_index(['Species','Kit'])
        self.kitDict = {x:kitDf.query("Species == @x").index.unique('Kit').tolist() for x in kitDf.index.unique('Species').tolist()}
        
        speciesVar = tk.StringVar(self)
        kitVar = tk.StringVar(self)
        
        kitMenu = tk.OptionMenu(mainWindow, kitVar, '')
        kitMenu.grid(row=1, column=4)
        
        def update_options(*args):
            kits = self.kitDict[speciesVar.get()]
            kitVar.set(kits[0])
            
            menu = kitMenu['menu']
            menu.delete(0, 'end')

            for kit in kits:
                menu.add_command(label=kit, command=lambda k=kit: kitVar.set(k))
            
        speciesVar.trace('w', update_options)
        speciesVar.set('Mouse')
        
        speciesMenu = tk.OptionMenu(mainWindow, speciesVar, *self.kitDict.keys())
        speciesMenu.grid(row=1, column=3)

        l4 = tk.Label(mainWindow, text='Cytokine kit used:').grid(row=0, column=4)
        
        tk.Button(mainWindow, text="Create new kit",command=lambda: master.switch_frame(KitCreationPage,folderName,expNum,ex_data,secondaryhomepage,bPage)).grid(row=3,column=4)
         
        def collectInputs():
            numCalibrationSamples = int(t1.get())
            initialStandardVolume = float(t2.get())
            kit = kitVar.get()
            sdf = float(t4.get())
            calibrationParameterDict = {'Number': numCalibrationSamples,'Volume': initialStandardVolume, 'Kit':[kit], 'SerialDilutionFactor':sdf, 'Species':speciesVar.get()}
            with open('misc'+dirSep+'CBAcalibrationParameters-'+folderName+'.json','w') as f:
                json.dump(calibrationParameterDict,f)
            master.switch_frame(secondaryhomepage,folderName,expNum,ex_data,bPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class KitCreationPage(tk.Frame):
    def __init__(self, master,folderName,expNum,ex_data,shp,bPage):
        tk.Frame.__init__(self, master)

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10) 
        
        tk.Label(mainWindow, text='Species: ').grid(row=0, column=0)
        tk.Label(mainWindow, text='Cytokine: ').grid(row=1, column=0)
        tk.Label(mainWindow, text='MW (Daltons): ').grid(row=2,column=0)
        tk.Label(mainWindow, text='Mass (pg): ').grid(row=3,column=0)

        #cytDf = pd.read_pickle(master.homedirectory+'misc'+dirSep+'cytokineMWDf.pkl').set_index('Cytokine',append=True)
        cytDf = pd.read_csv(master.homedirectory+'misc'+dirSep+'cytokineMWDf.csv').set_index(['Species','Cytokine'])
        self.cytokineDict = {x:cytDf.query("Species == @x").index.unique('Cytokine').tolist() for x in cytDf.index.unique('Species').tolist()}
        self.mwDict = {x+'--'+y:z for x,y,z in zip(list(cytDf.index.get_level_values('Species')),list(cytDf.index.get_level_values('Cytokine')),cytDf['MW'])}
        
        self.cb1 = Combobox(mainWindow, state="readonly", values=list(self.cytokineDict.keys()))
        self.cb1.current(0)
        self.cb1.grid(row=0,column=1)
        
        kitButtonWindow = tk.Frame(mainWindow)
        kitButtonWindow.grid(row=5,column=0,columnspan=2,pady=(20,0))
        
        #kitDf = pd.read_pickle(master.homedirectory+'misc'+dirSep+'kitDf.pkl')
        kitDf = pd.read_csv(master.homedirectory+'misc'+dirSep+'kitDf.csv').set_index(['Species','Kit'])
        species = self.cb1.get()
        self.kitNameCombo = Combobox(kitButtonWindow,values=kitDf.query("Species == @species").index.unique('Kit').tolist())
        tk.Label(kitButtonWindow,text='Kit: ').grid(row=0,column=0)
        self.kitNameCombo.grid(row=0,column=1)

        def populate_slave():
            self.cb2['values'] = self.cytokineDict[self.cb1.get()]
            self.cb2.current(0)
            populate_slave2()
            
            species = self.cb1.get()
            #kitDf = pd.read_pickle(master.homedirectory+'misc'+dirSep+'kitDf.pkl')
            kitDf = pd.read_csv(master.homedirectory+'misc'+dirSep+'kitDf.csv').set_index(['Species','Kit'])
            self.kitNameCombo['values'] = kitDf.query("Species == @species").index.unique('Kit').tolist() 
            
            return True #must return True or validatecommand will quit
        
        self.cb1.config(validate="focus", validatecommand=populate_slave)

        self.cb2 = Combobox(mainWindow)
        self.cb2.grid(column=1, row=1)
         
        self.mwEntry = tk.Entry(mainWindow,width=10)
        self.mwEntry.grid(row=2,column=1)
        
        self.massEntry = tk.Entry(mainWindow,width=5)
        self.massEntry.grid(row=3,column=1)
        self.massEntry.insert(0,str(10000))

        def populate_slave2():
            if self.cb1.get()+'--'+self.cb2.get() in self.mwDict:
                self.mwEntry.delete(0,tk.END)
                mw = self.mwDict[self.cb1.get()+'--'+self.cb2.get()]
                self.mwEntry.insert(0,str(mw))
            return True #must return True or validatecommand will quit

        self.cb2.config(validate="focus", validatecommand=populate_slave2)
        populate_slave()
        populate_slave2()
        
        #self.currentKitSpeciesList = []
        self.currentKitCytokineList = []
        self.currentKitMWList = []
        self.currentKitMassList = []
        def add_cytokine():
            #self.currentKitSpeciesList.append(self.cb1.get())
            if self.cb2.get() not in self.currentKitCytokineList:
                self.currentKitCytokineList.append(self.cb2.get())
                self.currentKitMWList.append(int(self.mwEntry.get()))
                self.currentKitMassList.append(int(self.massEntry.get()))
            else:
                cytIndex = self.currentKitCytokineList.index(self.cb2.get())
                self.currentKitCytokineList[cytIndex] = self.cb2.get()
                self.currentKitMWList[cytIndex] = int(self.mwEntry.get())
                self.currentKitMassList[cytIndex] = int(self.massEntry.get())

            print(self.cb1.get()+' '+self.cb2.get()+' with MW '+str(self.mwEntry.get())+' added\nCurrent cytokine list: '+','.join(self.currentKitCytokineList)+'\n')
        
        def remove_cytokine():
            #removedSpecies = self.currentKitSpeciesList.pop()
            removedSpecies = self.cb1.get() 
            removedCyt = self.currentKitCytokineList.pop()
            removedMW = self.currentMWList.pop()
            removedConc = self.currentConcList.pop()
            print(removedSpecies+' '+removedCyt+' with MW '+str(removedMW)+' removed\nCurrent cytokine list: '+','.join(self.currentKitCytokineList)+'\n')

        cytokineButtonWindow = tk.Frame(mainWindow)
        cytokineButtonWindow.grid(row=4,column=0,columnspan=2)
        
        tk.Button(cytokineButtonWindow, text="Add cytokine",command=lambda: add_cytokine()).grid(row=0,column=0)
        tk.Button(cytokineButtonWindow, text="Remove cytokine",command=lambda: remove_cytokine()).grid(row=0,column=1)

        kitButtonWindow2 = tk.Frame(mainWindow)
        kitButtonWindow2.grid(row=6,column=0,columnspan=2)
        def add_kit():
            #oldKitDf = pd.read_pickle(master.homedirectory+'misc'+dirSep+'kitDf.pkl')
            oldKitDf = pd.read_csv(master.homedirectory+'misc'+dirSep+'kitDf.csv').set_index(['Species','Kit'])
            listLength = len(self.currentKitCytokineList)
            newKitDf = pd.DataFrame({'Species':[self.cb1.get()]*listLength,'Kit':[self.kitNameCombo.get()]*listLength,'Cytokine':self.currentKitCytokineList,'MW':self.currentKitMWList,'Mass':self.currentKitMassList}).set_index(oldKitDf.index.names)
            kitDf = pd.concat([oldKitDf,newKitDf])
            tempDfList = []
            for s in sorted(kitDf.index.unique('Species').tolist()):
                sDf = kitDf.query("Species == @s")
                tempDf = pd.concat([sDf.query("Kit == @x") for x in sorted(sDf.index.unique('Kit').tolist())])
                tempDfList.append(tempDf)
            kitDf = pd.concat(tempDfList)
            additionalMessage = '\nThis kit contains these cytokines: \n'+','.join(self.currentKitCytokineList)
            answer = askyesno(title='Confirmation',message='Are you sure you want to add '+self.cb1.get()+' '+self.kitNameCombo.get()+' kit?'+additionalMessage)
            if answer:
                #kitDf.drop_duplicates().to_pickle(master.homedirectory+'misc'+dirSep+'kitDf.pkl')
                temp = kitDf.reset_index().drop_duplicates()
                dList = []
                for species in pd.unique(temp['Species']):
                    sDf  = temp.query("Species == @species")
                    for kit in pd.unique(sDf['Kit']):
                        kDf = sDf.query("Kit == @kit")
                        dList.append(kDf)
                temp = pd.concat(dList)
                temp.to_csv(master.homedirectory+'misc'+dirSep+'kitDf.csv',index=None)
                self.currentKitCytokineList = []
                self.currentKitMWList = []
                populate_slave()
                showinfo(title='Kit added',message=self.cb1.get()+' '+self.kitNameCombo.get()+' kit added!')

        def remove_kit():
            spec = self.cb1.get()
            k = self.kitNameCombo.get() 
            answer = askyesno(title='Confirmation', message='Are you sure you want to remove '+spec+' '+k+' kit?')
            if answer:
                #kitDf = pd.read_pickle(master.homedirectory+'misc'+dirSep+'kitDf.pkl')
                kitDf = pd.read_csv(master.homedirectory+'misc'+dirSep+'kitDf.csv').set_index(['Species','Kit'])
                #kitDf.query("Kit != @k").to_pickle(master.homedirectory+dirSep+'misc/kitDf.pkl')
                kitDf.query("Kit != @k").reset_index().to_csv(master.homedirectory+dirSep+'misc/kitDf.csv',index=None)
                populate_slave()
                showinfo(title='Kit removed',message=spec+' '+k+' kit removed!')

        tk.Button(kitButtonWindow2, text="Add kit",command=lambda: add_kit()).grid(row=0,column=0)
        tk.Button(kitButtonWindow2, text="Remove kit",command=lambda: remove_kit()).grid(row=0,column=1)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
         
        def collectInputs():
            answer = askyesno(title='Confirmation', message='Are you sure you want to exit cytokine kit creation?')
            if answer:
                master.switch_frame(shp,folderName,expNum,ex_data,bPage)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

def parseCytokineCSVHeaders(columns,cytokineMWDf,cytRenamingDict):
    #,Beads/IFNg | Geometric Mean (YG586-A),Beads/IL-2 | Geometric Mean (YG586-A),Beads/IL-4 | Geometric Mean (YG586-A),Beads/IL-6 | Geometric Mean (YG586-A),Beads/IL-10 | Geometric Mean (YG586-A),Beads/IL-17A | Geometric Mean (YG586-A),Beads/TNFa | Geometric Mean (YG586-A),
    newMultiIndexList = []
    for column in columns[1:-1]:
        populationNameVsStatisticSplit = column.split(' | ')
        if 'Barcode' not in populationNameVsStatisticSplit[0]:
            cytokine = populationNameVsStatisticSplit[0].split('/')[-1]
        else:
            cytokine = populationNameVsStatisticSplit[0].split('/')[-2]
        if cytokine in cytRenamingDict:
            cytokine = cytRenamingDict[cytokine]
        newMultiIndexList.append([cytokine])
    
    unparsedCytokines = []
    if type(cytokineMWDf) != list:
        kitCytokines = list(pd.unique(cytokineMWDf.reset_index()['Cytokine']))
        for cyt in newMultiIndexList:
            if cyt[0] not in kitCytokines:
                unparsedCytokines.append(cyt[0])

    return newMultiIndexList,unparsedCytokines

def performCommaCheck(fileName):
    with open('inputData'+dirSep+'bulkCSVFiles'+dirSep+fileName, 'r') as istr:
        with open('inputData'+dirSep+'bulkCSVFiles'+dirSep+'temp-'+fileName, 'w') as ostr:
            for line in istr:
                if line[-1] != ',':
                    line = line.rstrip('\n') + ','
                    print(line, file=ostr)
                else:
                    line = line.rstrip('\n')
                    print(line, file=ostr)
    os.remove('inputData'+dirSep+'bulkCSVFiles'+dirSep+fileName)
    os.rename('inputData'+dirSep+'bulkCSVFiles'+dirSep+'temp-'+fileName,'inputData'+dirSep+'bulkCSVFiles'+dirSep+fileName)

#Create Calibration Curves, obtain LODs (limits of detection) of experiment
def calibrateExperiment(folderName,secondPath,concUnit,concUnitPrefix,numberOfCalibrationSamples,initialStandardVolume,serialDilutionFactor,completeCytokineMWDf,cytRenamingDict={}):
    #Get cytokine calibration curve data
    tempExperimentParameters = {'overallPlateDimensions':[8,12]}
    calibrationFileNames = glob.glob('inputData'+dirSep+'bulkCSVFiles'+dirSep+'Calibration*')
    calibrationNames = []
    kitNames = []
    for calibrationFileName in calibrationFileNames:
        #performCommaCheck(calibrationFileName.split('/')[-1])
        newName = calibrationFileName.split('.')[0].split('_')[0].split(dirSep)[-1]
        kitNames.append(newName)
    sortedData,sortedFiles = cleanUpFlowjoCSV(kitNames,folderName,'cyt',tempExperimentParameters)
    for i,newName in enumerate(kitNames):
        if '-' in newName:
            newName2 = newName.split('-')[1]
        else:
            newName2 = newName
        kitNames[i] = newName2
    rsquaredList = []
    concLODList = []
    fittingParametersList = []
    cbaStandardsMFIList = []
    cbaPlotPointsMFIList = []
    cbaStandardsConcentrationList = []
    cbaPlotPointsConcentrationList = []

    numberOfPlotPoints = 101
    xaxistitle = 'Concentration of Cytokine Standards Standards ('+concUnitPrefix+')' 
    yaxistitle = 'GeoMFI'

    # Combine cytokine MWs into a dict
    completeCytokineMWDict = {x:y for x,y in zip(completeCytokineMWDf.reset_index()['Cytokine'],completeCytokineMWDf.reset_index()['MW'])}
    completeCytokineMassDict = {x:y for x,y in zip(completeCytokineMWDf.reset_index()['Cytokine'],completeCytokineMWDf.reset_index()['Mass'])}
    
    #TODO: Use fuzzy here to match mispelled cytokine names
    allCytokinesHaveMWInDict = True
    unparsedCytokinesBool = False
    for calibration in sortedData:
        cytokines,unparsedCytokines = parseCytokineCSVHeaders(calibration.columns,completeCytokineMWDf,cytRenamingDict)
        if len(unparsedCytokines) == 0:
            cytokines = [x[0] for x in cytokines]
            for cytokine in cytokines:
                if cytokine not in completeCytokineMWDict:
                    allCytokinesHaveMWInDict = False
        else:
            unparsedCytokinesBool = True
            break
    
    if not unparsedCytokinesBool:
        for calibration in sortedData:
            data = np.array(calibration.values[:,1:-1],dtype=float)
            cytokines,unparsedCytokines = parseCytokineCSVHeaders(calibration.columns,completeCytokineMWDf,cytRenamingDict)
            cytokines = [x[0] for x in cytokines]
            fittingParameters = np.zeros((data.shape[1],4))
            concLOD = np.zeros((data.shape[1],4))
            
            #masses in pg
            masses = np.array([completeCytokineMassDict[cyt] for cyt in cytokines])
            #conc in g/L
            initialConc = (masses*1e-12) /((initialStandardVolume*1e-3))
            
            #Calibration samples are always diluted by a factor of serialdilutionFactor (so with 12 calibration samples, the last sample is (serialDilutionFactor^-11) the concentration of the first, which is pure standard (2^0)
            x = np.linspace(-numberOfCalibrationSamples+1,0,numberOfCalibrationSamples)
            cbaStandardsConcentrations = np.fliplr(np.multiply(np.tile(initialConc, (x.shape[0],1)).T,np.power(serialDilutionFactor,np.tile(x, (initialConc.shape[0],1)))))
            #More x values along the above concentration bounds are sampled to use to construct calibration curve. Plot points are extended slightly at high range to allow visualization of upper LOD not accessible with experimental dilution
            x = np.linspace(-numberOfCalibrationSamples+1,4,numberOfPlotPoints)
            cbaStandardsConcentrationsPlotPoints = np.fliplr(np.multiply(np.tile(initialConc, (x.shape[0],1)).T,np.power(serialDilutionFactor,np.tile(x, (initialConc.shape[0],1)))))
            
            cbaStandardsConcentrationMatrix = np.zeros([len(cytokines),cbaStandardsConcentrations.shape[1]]) 
            cbaStandardsConcentrationPlotPointsMatrix = np.zeros([len(cytokines),cbaStandardsConcentrationsPlotPoints.shape[1]])
            cbaStandardsMFIMatrix = np.zeros([len(cytokines),cbaStandardsConcentrations.shape[1]]) 
            cbaStandardsMFIPlotPointsMatrix = np.zeros([len(cytokines),cbaStandardsConcentrationsPlotPoints.shape[1]])
            color_list = sns.color_palette(sns.color_palette(),len(cytokines))
            rsquared_kit = []
            for i,cytokineList in enumerate(cytokines):
                #amplitude bounded from range/2 to range*2, EC50 bounded from minimum to maximum standard concentration tested, Hill coefficient bounded from 0 to 2, Background bounded from 0 to minimum GFI*2
                lowerCurveFitBounds = [(np.max(data[:,i])-np.min(data[:,i]))/2,np.min(cbaStandardsConcentrations),0,0]
                upperCurveFitBounds = [(np.max(data[:,i])-np.min(data[:,i]))*2, np.max(cbaStandardsConcentrations), 2,np.min(data[:,i])*2]
                #use scipy curve fit to determine best hill equation fit for data, searching within the bounds given above
                popt,pcov = curve_fit(Hill, cbaStandardsConcentrations[i,:],np.log10(data[:,i]),sigma=np.log10(data[:,i]),bounds=(lowerCurveFitBounds,upperCurveFitBounds))
                rsquared = round(r_squared(cbaStandardsConcentrations[i,:],np.log10(data[:,i]),Hill,popt),3)
                rsquared_kit.append(rsquared)
                for j in range(len(popt)):  
                    #Convert just ec50 value to desired units (nM,uM etc) if cytokine has a molar mass in dict
                    if j == 1 and allCytokinesHaveMWInDict:
                        fittingParameters[i,j] = np.multiply(popt[j],(concUnit/completeCytokineMWDict[cytokine]))
                    #other values in 4 parameter logistic equation are tied to intensity y-value, which doesn't change, or are the hill coefficient, which is completely separate, so parameters are kept the same
                    else:
                        fittingParameters[i,j] = popt[j]
                
                #Convert x values of experimental data points and curve fit points to desired units (nM,uM,etc.)
                if allCytokinesHaveMWInDict:
                    cbaStandardsConcentrationMatrix[i,:] = np.multiply(cbaStandardsConcentrations[i,:],(concUnit/completeCytokineMWDict[cytokine]))
                    cbaStandardsConcentrationPlotPointsMatrix[i,:] = np.multiply(cbaStandardsConcentrationsPlotPoints[i,:],(concUnit/completeCytokineMWDict[cytokine]))
                else:
                    cbaStandardsConcentrationMatrix[i,:] = cbaStandardsConcentrations[i,:]
                    cbaStandardsConcentrationPlotPointsMatrix[i,:] = cbaStandardsConcentrationsPlotPoints[i,:]
                cbaStandardsMFIMatrix[i,:] = data[:,i]
                cbaStandardsMFIPlotPointsMatrix[i,:] = np.power(10,Hill(cbaStandardsConcentrationPlotPointsMatrix[i,:],*fittingParameters[i,:]))
                #Plot on log-log scale the experimental points and the curve fit line with previously determined curve fitting parameters
                #plt.loglog(cbaStandardsConcentrations,data[:,i],'o',color=color_list[i,:],label=listOfCytokines[i])
                #plt.loglog(cbaStandardsConcentrationsPlotPoints,np.power(10,Hill(convertedCBAStandardsPlotPoints,*fittingParameters[i,:])))
                #'_fit; R2 = '+str(rsquared)

                #Get LOD for each cytokine calibration curve (aka the linear range of the calibration curve)
                backgroundGFI = fittingParameters[i,3]
                amplitudeGFI = fittingParameters[i,0]
                
                #Approximate LOD by determining concentration values at LOD% and 1-LOD% (3% and 97%) of curve. Must be used on log10(curve), as calibration curve is plotted in logscale
                LODpercent = 0.03
                #LOD% more than background GFI used for lower LOD GFI
                lowerGFILOD = math.log10(10**((1+LODpercent)*math.log10(backgroundGFI)))
                #LOD% less than maximum GFI (Background + amplitude) used for upper LOD GFI
                upperGFILOD = math.log10(10**((1-LODpercent)*math.log10(amplitudeGFI+backgroundGFI)))
                #Log10(upper/lowerGFILOD) converted back into normal GFI by 10 to its power, then fed into inverse hill equation with current cytokine fitting parameters to obtain corresponding concentration values
                lowerConcLOD = InverseHill(lowerGFILOD,fittingParameters[i,:])
                upperConcLOD = InverseHill(upperGFILOD,fittingParameters[i,:])
                #Create dict with keys as cytokines, values as GFI/conc LODs
                concLOD[i,:] = np.array([10**lowerGFILOD,10**upperGFILOD,lowerConcLOD,upperConcLOD])
            flattenedMatrix = cbaStandardsMFIMatrix.flatten()
            reshapedMatrix = np.reshape(flattenedMatrix,(numberOfCalibrationSamples,len(cytokines)),order='F')
            flattenedMatrix2 = cbaStandardsMFIPlotPointsMatrix.flatten()
            reshapedMatrix2 = np.reshape(flattenedMatrix2,(numberOfPlotPoints,len(cytokines)),order='F')
            flattenedMatrix3 = cbaStandardsConcentrationMatrix.flatten()
            reshapedMatrix3 = np.reshape(flattenedMatrix3,(numberOfCalibrationSamples,len(cytokines)),order='F')
            flattenedMatrix4 = cbaStandardsConcentrationPlotPointsMatrix.flatten()
            reshapedMatrix4 = np.reshape(flattenedMatrix4,(numberOfPlotPoints,len(cytokines)),order='F')
            realCytokineList = cytokines
            dataValsList = []
            plotPointsList = []
            for j in range(1,numberOfCalibrationSamples+1):
                dataValsList.append([j])
            for j in range(1,numberOfPlotPoints+1):
                plotPointsList.append([j])
            dataValsIndex = pd.MultiIndex.from_tuples(dataValsList,names=['Standard'])
            plotPointsIndex = pd.MultiIndex.from_tuples(plotPointsList,names=['Standard'])
            currentCBAStandardsMFIDf = pd.DataFrame(reshapedMatrix,index=dataValsIndex,columns=realCytokineList)
            currentCBAPlotPointsMFIDf = pd.DataFrame(reshapedMatrix2,index=plotPointsIndex,columns=realCytokineList)
            currentCBAStandardsConcentrationDf = pd.DataFrame(reshapedMatrix3,index=dataValsIndex,columns=realCytokineList)
            currentCBAPlotPointsConcentrationDf = pd.DataFrame(reshapedMatrix4,index=plotPointsIndex,columns=realCytokineList)
           
            currentCBAStandardsMFIDf.columns.name = 'Cytokine' 
            currentCBAPlotPointsMFIDf.columns.name = 'Cytokine'
            currentCBAStandardsConcentrationDf.columns.name = 'Cytokine' 
            currentCBAPlotPointsConcentrationDf.columns.name = 'Cytokine'
            mic1 = pd.Index(cytokines, name='Cytokine')
            fittingParametersDf = pd.DataFrame(fittingParameters,index=mic1,columns=['Amplitude','EC50','HillCoeff','Background'])
            mic2 = pd.MultiIndex.from_tuples([['MFI','Lower'],['MFI','Upper'],['Concentration','Lower'],['Concentration','Upper']])
            concLODDf = pd.DataFrame(concLOD,index=mic1,columns=mic2)

            rsquaredDf = pd.DataFrame(rsquared_kit, index=mic1, columns=['R squared'])

            concLODList.append(concLODDf)
            fittingParametersList.append(fittingParametersDf)
            cbaStandardsMFIList.append(currentCBAStandardsMFIDf)
            cbaPlotPointsMFIList.append(currentCBAPlotPointsMFIDf)
            cbaStandardsConcentrationList.append(currentCBAStandardsConcentrationDf)
            cbaPlotPointsConcentrationList.append(currentCBAPlotPointsConcentrationDf)
            rsquaredList.append(rsquaredDf)

        #fullFittingParametersDf = pd.concat(fittingParametersList,keys=kitNames,names=['Kit Name'])
        fullConcLODDf = pd.concat(concLODList)
        fullFittingParametersDf = pd.concat(fittingParametersList)
        fullRsquaredDf = pd.concat(rsquaredList)
        #fullConcLODDf = pd.concat(concLODList)
        
        fullCBAStandardsMFIDf = pd.concat(cbaStandardsMFIList,keys=kitNames,names=['Kit Name'],axis=1)
        fullCBAPlotPointsMFIDf = pd.concat(cbaPlotPointsMFIList,keys=kitNames,names=['Kit Name'],axis=1)
        fullCBAStandardsConcentrationDf = pd.concat(cbaStandardsConcentrationList,keys=kitNames,names=['Kit Name'],axis=1)
        fullCBAPlotPointsConcentrationDf = pd.concat(cbaPlotPointsConcentrationList,keys=kitNames,names=['Kit Name'],axis=1)
        fullCBAStandardsList = [fullCBAStandardsMFIDf.stack().stack(),fullCBAStandardsConcentrationDf.stack().stack()]
        fullCBAPlotPointsList = [fullCBAPlotPointsMFIDf.stack().stack(),fullCBAPlotPointsConcentrationDf.stack().stack()]
        fullCBAStandardsDf = pd.concat(fullCBAStandardsList,axis=1,keys=[yaxistitle,xaxistitle])
        fullCBAPlotPointsDf = pd.concat(fullCBAPlotPointsList,axis=1,keys=[yaxistitle,xaxistitle])
         
        plottingPointsDf = fullCBAPlotPointsDf.reset_index()
        plottingStandardsDf = fullCBAStandardsDf.reset_index()
        
        numCyt = len(pd.unique(plottingPointsDf['Cytokine']))
        if numCyt <= 13:
            fullpalette = sns.color_palette(sns.color_palette(cc.glasbey),numCyt)
            g = sns.relplot(data=plottingPointsDf,x=xaxistitle,y=yaxistitle,hue='Cytokine',col='Kit Name',kind='line',col_order=pd.unique(plottingPointsDf['Kit Name']),hue_order=pd.unique(plottingPointsDf['Cytokine']),height=7,palette=fullpalette)
            #Plot vertical lines at lower and upper concentration limits of detection
            colorDict = {}
            for j,cytokine in enumerate(pd.unique(plottingPointsDf['Cytokine'])):
                colorDict[cytokine] = fullpalette[j]
            for axis,kitName in zip(g.axes.flat,pd.unique(plottingPointsDf['Kit Name'])):
                currentpalette = []
                for cytokine in pd.unique(plottingStandardsDf.query("`Kit Name` == @kitName")['Cytokine']):
                    currentColor = colorDict[cytokine]
                    currentpalette.append(currentColor)
                    cytokineLODValues = fullConcLODDf.loc[cytokine,:]['Concentration']
                    axis.axvline(x=cytokineLODValues['Lower'],color=currentColor,linestyle=':')
                    axis.axvline(x=cytokineLODValues['Upper'],color=currentColor,linestyle=':')
                g2 = sns.scatterplot(data=plottingStandardsDf[plottingStandardsDf['Kit Name'] == kitName],x=xaxistitle,y=yaxistitle,hue='Cytokine',ax=axis,legend=False,palette=currentpalette)
                axis.set_xscale('log')
                axis.set_yscale('log')
                if not allCytokinesHaveMWInDict:
                    axis.set_ylabel('Concentration of Cytokine Standards (pg/mL)')
                    concUnitPrefix = 'pg_mL'
            plt.savefig('plots/calibrationCurves-'+folderName+'-'+concUnitPrefix+'.png',bbox_inches='tight', dpi=200)
        else:
            fullpalette = sns.color_palette(sns.color_palette(),len(pd.unique(plottingPointsDf['Kit Name'])))
            colorDict = {}
            g = sns.relplot(data=plottingPointsDf,x=xaxistitle,y=yaxistitle,hue='Kit Name',col='Cytokine',kind='line',hue_order=pd.unique(plottingPointsDf['Kit Name']),col_order=pd.unique(plottingPointsDf['Cytokine']),col_wrap=math.floor(np.sqrt(numCyt)),height=12,palette=fullpalette,facet_kws={'sharey':False, 'sharex':False})
            for j,kit in enumerate(pd.unique(plottingPointsDf['Kit Name'])):
                    colorDict[kit] = fullpalette[j]
            for axis,cytName in zip(g.axes.flat,pd.unique(plottingPointsDf['Cytokine'])):
                currentpalette = []
                for kit in pd.unique(plottingStandardsDf.query("Cytokine == @cytName")['Kit Name']):
                    currentColor = colorDict[kit]
                    currentpalette.append(currentColor)
                    cytokineLODValues = fullConcLODDf.loc[cytName,:]['Concentration']
                    axis.axvline(x=cytokineLODValues['Lower'],color=currentColor,linestyle=':')
                    axis.axvline(x=cytokineLODValues['Upper'],color=currentColor,linestyle=':')
                g2 = sns.scatterplot(data=plottingStandardsDf[plottingStandardsDf['Cytokine'] == cytName],x=xaxistitle,y=yaxistitle,hue='Kit Name',ax=axis,legend=False)
                axis.set_xscale('log')
                axis.set_yscale('log')
                if not allCytokinesHaveMWInDict:
                    axis.set_ylabel('Concentration of Cytokine Standards (pg/mL)')
                    concUnitPrefix = 'pg_mL'
            plt.tight_layout()
            g.fig.savefig('plots'+dirSep+'calibrationCurves-'+folderName+'-'+concUnitPrefix+'.png',bbox_inches='tight', dpi=200)
        #Save fitting parameters and LOD for curve fit for each cytokine
        with open('misc'+dirSep+'fittingParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "wb") as f:
            pickle.dump(fullFittingParametersDf, f)
        with open('misc'+dirSep+'LODParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "wb") as f:
            pickle.dump(fullConcLODDf, f)
        with open('misc/rsquaredValues-'+folderName+'-'+concUnitPrefix+'.pkl', "wb") as f:
            pickle.dump(fullRsquaredDf, f)
    return unparsedCytokines

def createCytokineDataFrame(folderName,finalDataFrame,concUnitPrefix):
         
        columnName = finalDataFrame.columns.name
        with open('outputData'+dirSep+'pickleFiles'+dirSep+'cytokineGFIPickleFile-'+folderName+'.pkl', "wb") as f:
            pickle.dump(finalDataFrame, f)
        
        fittingParameters = pickle.load(open('misc'+dirSep+'fittingParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "rb"))
        LODParameters = pickle.load(open('misc'+dirSep+'LODParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "rb"))
        #Begin converting GFI dataframe into corresponding concentration dataframe
        concentrationList = []
        #Step through dataframe one cytokine at a time
        idx = pd.IndexSlice
        for cytokine in pd.unique(finalDataFrame.index.get_level_values(0)):
            #Retrieve LODs for current cytokine (from constructed calibration curve)
            lowerGFILOD = LODParameters.loc[cytokine,idx['MFI','Lower']]
            upperGFILOD = LODParameters.loc[cytokine,idx['MFI','Upper']]
            lowerConcLOD = LODParameters.loc[cytokine,idx['Concentration','Lower']]
            upperConcLOD = LODParameters.loc[cytokine,idx['Concentration','Upper']]
            smallConcentrationMatrix = np.zeros(finalDataFrame.loc[cytokine].shape)
            #Loop through every value in current cytokine's portion of the dataframe
            for i in range(0,finalDataFrame.loc[cytokine].values.shape[0]):
                for j in range(0,finalDataFrame.loc[cytokine].values.shape[1]):
                    currentGFIval = finalDataFrame.loc[cytokine].values[i,j]
                    if currentGFIval > upperGFILOD: #If intensity is greater than upper GFI LOD
                        currentConcVal = upperConcLOD #Concentration is equal to upper concentration LOD
                    elif currentGFIval <= upperGFILOD and currentGFIval >= lowerGFILOD: #if intensity is between upper and lower GFI LODs
                        currentConcVal = InverseHill(np.log10(currentGFIval),fittingParameters.loc[cytokine,:].values) #Use previous hill fit parameters for the cytokine to obtain concentration
                    else: #If intensity is less than background GFI LOD
                        currentConcVal = lowerConcLOD #Concentration is equal to lower concentration LOD
                    smallConcentrationMatrix[i,j] = currentConcVal
            concentrationList.append(smallConcentrationMatrix)

        concentrationMatrix = np.vstack(concentrationList)
        finalDataFrameConcentration = pd.DataFrame(concentrationMatrix,index=finalDataFrame.index,columns=finalDataFrame.columns)
        finalDataFrameConcentration.columns.name = columnName 
        return finalDataFrameConcentration



#!/usr/bin/env python3
import pickle
import pandas as pd
import numpy as np
import tkinter as tk
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'
from plateypus.dataprocessing.miscFunctions import setMaxWidth

expParamDict = {'cyt':'cyt','cell':'cell','prolif':'cell','singlecell':'cell'}

class TubeLayoutPage(tk.Frame):
    def __init__(self,master,folderName,levelLabelDict,numSamples,dataType,secondaryhomepage,backPage):
        allLevelNames = list(levelLabelDict.keys())
        conditionLevelValues = [levelLabelDict[x] for x in list(levelLabelDict.keys())[:-1]]
        columnLevelValues = levelLabelDict[list(levelLabelDict.keys())[-1]]
        numericList = [False]*len(conditionLevelValues) + [True]

        tk.Frame.__init__(self, master)
        
        labelWindow = tk.Frame(self)
        labelWindow.pack()
        l1 = tk.Label(labelWindow, text="Assign level values to each sample:", font='Helvetica 18 bold').pack()

        """BEGIN TEMP SCROLLBAR CODE"""
        labelWindow1 = tk.Frame(self)
        labelWindow1.pack(side=tk.TOP,padx=10,fill=tk.X,expand=True)
        
        #Make canvas
        w1 = tk.Canvas(labelWindow1, width=1500, height=400,background="white", scrollregion=(0,0,3000,33*numSamples))

        #Make scrollbar
        scr_v1 = tk.Scrollbar(labelWindow1,orient=tk.VERTICAL)
        scr_v1.pack(side=tk.RIGHT,fill=tk.Y)
        scr_v1.config(command=w1.yview)
        #Add scrollbar to canvas
        w1.config(yscrollcommand=scr_v1.set)
        
        scr_v2 = tk.Scrollbar(labelWindow1,orient=tk.HORIZONTAL)
        scr_v2.pack(side=tk.BOTTOM,fill=tk.X)
        scr_v2.config(command=w1.xview)
        w1.config(xscrollcommand=scr_v2.set)

        w1.pack(fill=tk.BOTH,expand=True)
        #Make and add frame for widgets inside of canvas
        #canvas_frame = tk.Frame(w1)
        mainWindow = tk.Frame(w1)
        mainWindow.pack()
        w1.create_window((0,0),window=mainWindow, anchor = tk.NW)
        """END TEMP SCROLLBAR CODE"""
 
        #mainWindow = tk.Frame(self)
        #mainWindow.pack(fill=tk.BOTH, expand=True)
        #scrollable_body = Scrollable(mainWindow, width=100)
        
        #Automatically import file names into layout dict if excel/csv template in correct folder
        if 'sampleNameFile.xlsx' in os.listdir('misc') or 'sampleNameFile.csv' in os.listdir('misc'):
            if 'sampleNameFile.xlsx' in os.listdir('misc'): 
                sampleNameDf = pd.read_excel('misc'+dirSep+'sampleNameFile.xlsx')
            else:
                sampleNameDf = pd.read_csv('misc'+dirSep+'sampleNameFile.csv')
        else:
            sampleNameDf = []

        #Should be in this format: headers are: sampleFileName Level0Name Level1Name
        fcsFiles = []
        if 'A1_cell.csv' not in os.listdir('inputData'+dirSep+'bulkCSVFiles'+dirSep):
            for fcsName in os.listdir('inputData'+dirSep+'fcsFiles'):
                if '.fcs' in fcsName:
                    fcsFiles.append(fcsName)
            if len(fcsFiles) == 0:
                fcsFiles = ['                 ']
        else:
            bulkStatFile = pd.read_csv('inputData'+dirSep+'bulkCSVFiles'+dirSep+'A1_cell.csv')
            for row in range(bulkStatFile.shape[0]):
                if bulkStatFile.iloc[row,0] not in ['Mean','SD']:
                    fcsFiles.append(bulkStatFile.iloc[row,0])

        fcsMenuList = []
        fcsVarList = []
        includeCheckboxList = []
        includeBoolVarList = []
        sampleDropdownList = []
        sampleParameterList = []
        
        tk.Label(mainWindow,text='FileName: ').grid(row=0,column=1)
        tk.Label(mainWindow,text='Include?').grid(row=0,column=2)
        for i,level in enumerate(allLevelNames):
            tk.Label(mainWindow,text=level+': ').grid(row=0,column=i+3)
        
        print(numSamples)

        for sample in range(numSamples):
            tk.Label(mainWindow,text='Sample '+str(sample+1)+': ').grid(row=sample+1,column=0)
            fcsVar = tk.StringVar()
            #fcsMenu = tk.OptionMenu(scrollable_body,fcsVar,*fcsFiles)
            fcsMenu = tk.OptionMenu(mainWindow,fcsVar,*fcsFiles)
            fcsMenu.grid(row=sample+1,column=1)
            setMaxWidth(fcsFiles,fcsMenu)
            
            fcsMenuList.append(fcsMenu)
            fcsVarList.append(fcsVar)
            includeBoolVar = tk.BooleanVar(value=True)
            includeCheckbutton = tk.Checkbutton(mainWindow,variable=includeBoolVar)
            includeCheckbutton.grid(row=sample+1,column=2)
            includeCheckboxList.append(includeCheckbutton)
            includeBoolVarList.append(includeBoolVar)
            
            currentSampleDropdownList = []
            currentSampleParameterList = []
            for i,level in enumerate(allLevelNames):
                if level != 'Time':
                    levelValues = conditionLevelValues[i]
                else:
                    levelValues = columnLevelValues
                levelValueVar = tk.StringVar()
                #levelValueMenu = tk.OptionMenu(scrollable_body,levelValueVar,*levelValues)
                levelValueMenu = tk.OptionMenu(mainWindow,levelValueVar,*levelValues)
                if len(levelValues) == 1:
                    levelValueVar.set(levelValues[0])
                else:
                    if not isinstance(sampleNameDf,list) and level in sampleNameDf.columns:
                       fcsVar.set(sampleNameDf.iloc[sample,0])
                       levelValueVar.set(sampleNameDf.iloc[sample,:][level])
                levelValueMenu.grid(row=sample+1,column=i+3)
                setMaxWidth(levelValues,levelValueMenu)
                currentSampleDropdownList.append(levelValueMenu)
                currentSampleParameterList.append(levelValueVar)
            sampleDropdownList.append(currentSampleDropdownList)
            sampleParameterList.append(currentSampleParameterList)

        #scrollable_body.update() 
        
        def collectInputs():
            fullSampleList = []
            indexTuples = []
            for i,sample in enumerate(fcsVarList):
                if includeBoolVarList[i].get():
                    fullSampleList.append(sample.get())
                    sampleTuple = []
                    for j,sampleParameter in enumerate(sampleParameterList[i]):
                        if numericList[j]:
                            sampleTuple.append(float(sampleParameter.get()))
                        else:
                            sampleTuple.append(sampleParameter.get())
                    indexTuples.append(sampleTuple)
            mi = pd.MultiIndex.from_tuples(indexTuples,names=allLevelNames)
            data = fullSampleList 
            df1 = pd.Series(data,index=mi)
            df1.to_pickle('misc'+dirSep+'tempDf.pkl')
            df2 = df1.unstack('Time')
            
            #Reindex df after unstacking by original order
            xsdf = df1.xs([columnLevelValues[0]],level=['Time'])
            emptydf = np.empty(df2.shape,dtype='object')
            for row in range(emptydf.shape[0]):
                for col in range(emptydf.shape[1]):
                    timepoint = df2.columns[col]
                    if not isinstance(xsdf.index.tolist()[row],str) and not isinstance(xsdf.index.tolist()[row],float):
                        conditions = list(xsdf.index.tolist()[row])
                    else:
                        conditions = [xsdf.index.tolist()[row]]
                    emptydf[row,col] = df2.loc[tuple(conditions),timepoint]
            reindexedDf = pd.DataFrame(emptydf,index=xsdf.index,columns=df2.columns)
            reindexedDf.columns.name = 'Time'
            
            if dataType == 'both':
                dataTypes = ['cell','cyt']
            else:
                dataTypes = [dataType]
            for dt in dataTypes:
                with open('misc'+dirSep+'tubeLayout-'+folderName+'-'+dt+'.pkl','wb') as f:
                    pickle.dump(reindexedDf,f)
            
            master.switch_frame(secondaryhomepage,folderName,backPage)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(secondaryhomepage,folderName,backPage)).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)



#!/usr/bin/env python3
import os,itertools,shutil
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from operator import itemgetter
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'
import plateypus.plotting.facetPlot1D as fp1D
import plateypus.plotting.facetPlotCategorical as fpCategorical
import plateypus.plotting.facetPlot2D as fp2D
import plateypus.plotting.facetPlot3D as fp3D
from plateypus.dataprocessing.miscFunctions import sortSINumerically

idx = pd.IndexSlice
plotFolderName = 'plots'
def produceSubsettedDataFrames(fulldf,withinFigureBoolean,specificValueBooleanList,trueLabelDict):
    fulldf = fulldf.astype(float)
    
    #Get all possible subsetted indices
    figureSubsettedLevelValues = []
    withinFigureSubsettedLevelValues = []
    figureSubsettingLevels = []
    figureLevelNames = []
    figureLevelIndices = []
    levelValuesPlottedIndividually = []
    for levelIndex,currentLevelName in enumerate(fulldf.index.names):
        #Event level always has every value included (single cell)
        #Otherwise, check which level values in the level were selected by the user
        if currentLevelName != 'Event':
            #currentLevelValues = pd.unique(fulldf.index.get_level_values(currentLevelName))
            currentLevelValues = trueLabelDict[currentLevelName]
            levelValues = []
            if (currentLevelName == 'Marker' or currentLevelName == 'Markers') and 'Event' in fulldf.index.names:
                realLevelIndex = levelIndex - 1
            else:
                realLevelIndex = levelIndex
            #Go through each level value in the level; regardless of figure selection status, and add based on level value selection status
            for levelValue,specificBoolean in zip(currentLevelValues,specificValueBooleanList[realLevelIndex]):
                if specificBoolean:
                    levelValues.append(levelValue)
            #If we will include this level within the figure
            if withinFigureBoolean[realLevelIndex]:
                withinFigureSubsettedLevelValues.append(levelValues)
                if len(levelValues) == len(currentLevelValues):
                    for levelValue in levelValues:
                        levelValuesPlottedIndividually.append(levelValue)
            #Only need to add level values to figure list; will be xs'd out of the full dataframe in the subsetted, within figure dataframes
            else:
                figureSubsettedLevelValues.append(levelValues)
                figureLevelNames.append(currentLevelName)
                figureLevelIndices.append(levelIndex)
    if len(figureLevelIndices) > 0:
        #Get all row level values present in the dataframe
        allPossibleSubsettingCombos = itertools.product(*figureSubsettedLevelValues)
        if 'Event' not in fulldf.index.names:
            rowList = []
            for row in range(fulldf.shape[0]):
                allCurrentLevelValues = fulldf.iloc[row,:].name
                currentLevelValues = itemgetter(*figureLevelIndices)(allCurrentLevelValues)
                if not isinstance(currentLevelValues,tuple):
                    currentLevelValues = tuple([currentLevelValues])
                rowList.append(currentLevelValues)
            actualSubsettingCombos = []
            #From original dataframe; select all rows that appear in the all possible combination list 
            for subsettingCombo in allPossibleSubsettingCombos:
                if subsettingCombo in rowList:
                    actualSubsettingCombos.append(subsettingCombo) 
        else:
            rowList = []
            nonEventLevelList = []
            for level in fulldf.index.names:
                if level != 'Event':
                    nonEventLevelList.append(level)
            noEventDf = fulldf.groupby(nonEventLevelList).first()
            for row in range(noEventDf.shape[0]):
                allCurrentLevelValues = noEventDf.iloc[row,:].name
                figureLevelIndices = [max(min(x, len(fulldf.index.names)-2), 0) for x in figureLevelIndices]
                currentLevelValues = itemgetter(*figureLevelIndices)(allCurrentLevelValues)
                if not isinstance(currentLevelValues,tuple):
                    currentLevelValues = tuple([currentLevelValues])
                rowList.append(currentLevelValues)
            actualSubsettingCombos = []
            #From original dataframe; select all rows that appear in the all possible combination list 
            for subsettingCombo in allPossibleSubsettingCombos:
                if subsettingCombo in rowList:
                    actualSubsettingCombos.append(subsettingCombo) 
            #actualSubsettingCombos = allPossibleSubsettingCombos
        #Use these levels to cross section the fulldf, generating a list of subsetted dfs that will each have their own figure
        allPossibleSubsettedDfList = []
        for actualSubsettingCombo in actualSubsettingCombos:
            possibleSubsettedDf = fulldf.xs(actualSubsettingCombo, level=figureLevelNames)
            allPossibleSubsettedDfList.append(possibleSubsettedDf)
    else:
        actualSubsettingCombos = ['All']
        allPossibleSubsettedDfList = [fulldf]
    actualLevelValueDfList = []
    #Go through each subsetteddf, and only grab rows with level values that are selected
    for possibleSubsettedDf in allPossibleSubsettedDfList:
        allPossibleLevelValueCombos = itertools.product(*withinFigureSubsettedLevelValues)
        if 'Event' not in fulldf.index.names:
            rowList = []
            for row in range(possibleSubsettedDf.shape[0]):
                allCurrentLevelValues = possibleSubsettedDf.iloc[row,:].name
                rowList.append(allCurrentLevelValues)
            actualLevelValueCombos = []
            #From original dataframe; select all rows that appear in the all possible level value combination list 
            levelValueRowList = []
            for levelValueCombo in allPossibleLevelValueCombos:
                if levelValueCombo in rowList:
                    indices = [i for i, x in enumerate(rowList) if x == levelValueCombo]
                    levelValueRowList+=indices
            actualLevelValueDf = possibleSubsettedDf.iloc[levelValueRowList,:]
            actualLevelValueDfList.append(actualLevelValueDf)
        else:
            unstackedDf = possibleSubsettedDf.unstack('Event')
            allLevelValues = []
            for row in range(unstackedDf.shape[0]):
                allCurrentLevelValues = unstackedDf.iloc[row,:].name
                allLevelValues.append(allCurrentLevelValues)
            realLevelValueCombos = []
            for possibleLevelValueCombo in allPossibleLevelValueCombos:
                if possibleLevelValueCombo in allLevelValues:
                    realLevelValueCombos.append(possibleLevelValueCombo)
            subsettingList = list(map(list, zip(*realLevelValueCombos)))
            realSubsettingList = []
            for subset in subsettingList:
                newSubset = list(pd.unique(subset))
                realSubsettingList.append(newSubset)
            actualSubsettingList = []
            realsubset = 0
            for subset,levelName in enumerate(possibleSubsettedDf.index.names):
                if levelName == 'Event':
                    actualSubsettingList.append(slice(None))
                else:
                    actualSubsettingList.append(realSubsettingList[realsubset])
                    realsubset+=1
            actualLevelValueDf = possibleSubsettedDf.loc[tuple(actualSubsettingList),:]
            actualLevelValueDfList.append(actualLevelValueDf)
    if not isinstance(actualLevelValueDfList,list):
        actualLevelValueDfList = [actualLevelValueDfList]
    if not isinstance(actualSubsettingCombos,list):
        actualSubsettingCombos = ['All']

    #Remove columns from non postprocessed data
    if 'Dimension' not in fulldf.columns[0]:
        for i in range(len(actualLevelValueDfList)):
            actualLevelValueDfList[i].columns.name = ''
    return actualLevelValueDfList,actualSubsettingCombos,figureLevelNames,levelValuesPlottedIndividually

def createFacetPlotName(folderName,dataType,plotType,subPlotType,legendParameterToLevelNameDict,subsettedDfTitle,levelsPlottedIndividually,useModifiedDf,plotOptions):
    delimiter1 = '-'
    delimiter2 = ','
    
    legendParameterString = delimiter2.join(list(legendParameterToLevelNameDict.keys()))

    flattened_list = []
    for val in legendParameterToLevelNameDict.values():
        if isinstance(val, (list,)):
            for val2 in val:
                flattened_list.append(val2)
        else:
            flattened_list.append(val)

    levelNameString = delimiter2.join(flattened_list)
    figureLevelNameString = delimiter2.join(list(map(str,subsettedDfTitle)))

    if len(levelsPlottedIndividually) == 0:
        individualLevelString = 'all'
    else:
        individualLevelString = delimiter2.join(list(map(str,levelsPlottedIndividually)))
    if useModifiedDf:
        modifiedString = '-modified'
    else:
        modifiedString = ''

    axisScalingStringList = []
    for axis in plotOptions:
        if 'X' in axis:
            if plotOptions[axis]['axisScaling'] == 'Logarithmic':
                axisScalingStringList.append('logX')
            elif plotOptions[axis]['axisScaling'] == 'Biexponential':
                axisScalingStringList.append('biexpX')
            else:
                axisScalingStringList.append('linX')
        else:
            if plotOptions[axis]['axisScaling'] == 'Logarithmic':
                axisScalingStringList.append('logY')
            elif plotOptions[axis]['axisScaling'] == 'Biexponential':
                axisScalingStringList.append('biexpY')
            else:
                axisScalingStringList.append('linY')
    axisScalingString = delimiter2.join(axisScalingStringList)

    if len(subsettedDfTitle) == 0 or subsettedDfTitle == 'All':
        initialString = delimiter1.join([subPlotType,dataType,folderName,legendParameterString,levelNameString,axisScalingString])
    else:
        initialString = delimiter1.join([subPlotType,dataType,folderName,legendParameterString,levelNameString,figureLevelNameString,axisScalingString])
    fullTitleString = initialString+modifiedString
    if dirSep in fullTitleString:
        fullTitleString = fullTitleString.replace(dirSep, "_")
    if '.' in fullTitleString:
        fullTitleString = fullTitleString.replace(".", "_")
    if ' ' in fullTitleString:
        fullTitleString = fullTitleString.replace(" ", "_")
    return fullTitleString

def subsetOriginalLevelValueOrders(unsubsettedOrders,subsettedDf,parameter):
    subsettedValues = list(pd.unique(subsettedDf.index.get_level_values(parameter)))
    subsettedOrders = []
    for unsubsettedValue in unsubsettedOrders:
        if unsubsettedValue in subsettedValues:
            subsettedOrders.append(unsubsettedValue)
    return subsettedOrders

def plotFacetedFigures(folderName,plotType,subPlotType,dataType,subsettedDfList,subsettedDfListTitles,figureParameters,levelsPlottedIndividually,useModifiedDf,fulldf,plotOptions,legendParameterToLevelNameDict,addDistributionPoints,alternateTitle='',originalLevelValueOrders = {},subfolderName='',context='notebook',height=3,aspect=1,titleBool='yes',colwrap=5,legendBool='yes',cmap='',outlierZScore=3,plotAllVar=True,titleAdjust='',plotSpecificDict={}):
    sns.set_context(context)
    sns.set_palette(sns.color_palette())
    global hei,asp,titleVar,legendVar
    hei = height
    asp = aspect
    titleVar = titleBool
    legendVar = legendBool
    zScoreCutoff = 3
    #Has issues with repeated values (aka CD54 shows up in TCells and APCs)
    for subsettedDf,subsettedDfTitle in zip(subsettedDfList,subsettedDfListTitles):
        #if outlierBool == 'yes':
        #    beforeOutlierRemoval = subsettedDf.shape[0]
        #    if plotOptions['Y']['axisScaling'] == 'Logarithmic':
        #        minVal = min(subsettedDf.values)
        #        if minVal <= 0:
        #            newSubsettedDf = subsettedDf.iloc[:,:]+abs(minVal)+1
        #        else:
        #            newSubsettedDf = subsettedDf.copy()
        #        newSubsettedDf = np.log10(newSubsettedDf)
        #        subsettedDf = subsettedDf[(np.abs(stats.zscore(newSubsettedDf)) < outlierZScore).all(axis=1)]
        #    else:
        #        subsettedDf = subsettedDf[(np.abs(stats.zscore(subsettedDf)) < outlierZScore).all(axis=1)]
        #    afterOutlierRemoval = subsettedDf.shape[0]
        #    print(str(beforeOutlierRemoval-afterOutlierRemoval)+' outliers removed!')
            
        #Assign all levels to plot parameters in catplot/relplot; reassign x/y axis level names to desired x/y axis titles
        kwargs = {}
        facetgridkwargs = {}
        for parameter in legendParameterToLevelNameDict:
            if parameter == 'Y Axis Values':
                if subPlotType == 'heatmap':
                    kwargs['y'] = legendParameterToLevelNameDict[parameter]
            else:
                currentLevel = legendParameterToLevelNameDict[parameter]
            if parameter == 'Color':
                kwargs['hue'] = currentLevel
                if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                    kwargs['hue_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['hue']],subsettedDf,kwargs['hue'])
            elif parameter == 'Size':
                kwargs['size'] = currentLevel
                if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                    kwargs['size_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['size']],subsettedDf,kwargs['size'])
            elif parameter == 'Marker':
                kwargs['style'] = currentLevel
            elif parameter == 'Order':
                subsettedDf.index.set_names(plotOptions['X']['axisTitle'],level=currentLevel,inplace=True)
                kwargs['x'] = plotOptions['X']['axisTitle']
                if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                    kwargs['order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['x']],subsettedDf,kwargs['x'])
            elif parameter == 'Column':
                kwargs['col'] = currentLevel
                unorderedLevelValues = list(pd.unique(subsettedDf.index.get_level_values(currentLevel)))
                if 'Concentration' in currentLevel and 'M' in unorderedLevelValues[0]:
                    levelValues = sortSINumerically(unorderedLevelValues,True,True)[0]
                    kwargs['col_order'] = levelValues 
                else:
                    if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                        kwargs['col_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['col']],subsettedDf,kwargs['col'])
                    else:
                        kwargs['col_order'] = unorderedLevelValues
                facetgridkwargs['col'] = kwargs['col']
                facetgridkwargs['col_order'] = kwargs['col_order']
            elif parameter == 'Row':
                kwargs['row'] = currentLevel
                unorderedLevelValues = list(pd.unique(subsettedDf.index.get_level_values(currentLevel)))
                if 'Concentration' in currentLevel and 'M' in unorderedLevelValues[0]:
                    levelValues = sortSINumerically(unorderedLevelValues,True,True)[0]
                    kwargs['row_order'] = levelValues 
                else:
                    levelValues = unorderedLevelValues
                    if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                        kwargs['row_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['row']],subsettedDf,kwargs['row'])
                    else:
                        kwargs['row_order'] = unorderedLevelValues 
                facetgridkwargs['row'] = kwargs['row']
                facetgridkwargs['row_order'] = kwargs['row_order']
            elif parameter == 'X Axis Values':
                if len(plotOptions['X']['axisTitle']) > 1 and dataType != 'dr':
                    subsettedDf.index.set_names(plotOptions['X']['axisTitle'],level=currentLevel,inplace=True)
                    kwargs['x'] = plotOptions['X']['axisTitle']
                else:
                    kwargs['x'] = currentLevel
            else:
                pass
        #Assign y axis (or color bar axis) parameter
        #if subPlotType not in ['kde','histogram']:
        if dataType == 'dr':
            kwargs['y'] = plotOptions['Y']['axisTitle']
        else:
            if subPlotType != 'heatmap':
                if 'Statistic' in figureParameters:
                    subsettedDf.columns = [subsettedDfTitle[figureParameters.index('Statistic')]]
                    kwargs['y'] = subsettedDfTitle[figureParameters.index('Statistic')]
                else:
                    subsettedDf.columns = [plotOptions['Y']['axisTitle']]
                    kwargs['y'] = plotOptions['Y']['axisTitle']
            else:
                if 'Statistic' in figureParameters:
                    subsettedDf.columns = [subsettedDfTitle[figureParameters.index('Statistic')]]
                    kwargs['z'] = subsettedDfTitle[figureParameters.index('Statistic')]
                else:
                    subsettedDf.columns = [plotOptions['Colorbar']['axisTitle']]
                    kwargs['z'] = plotOptions['Colorbar']['axisTitle']
        
        if dataType == 'singlecell':
            plottingDf = subsettedDf.copy()
            #plottingDf = subsettedDf.stack().to_frame('GFI')
        else:
            plottingDf = subsettedDf.copy()
        #Converts wide form dataframe into long form required for cat/relplot
        plottingDf = plottingDf.reset_index()
        #Use plot options file to initialize numeric x axis ordering
        #NEED TO GET WORKING WITH HEATMAPS
        if subPlotType != 'heatmap':
            if plotType != '1d':
                if plotOptions['X']['numeric']:
                    currentLevelValues = list(plottingDf[kwargs['x']])
                    sortedOldLevelValues,newLevelValues = sortSINumerically(currentLevelValues,False,True)
                    #Need to interpret parenthetical units for x to get 1e9
                    if 'M)' in kwargs['x']:
                        s = kwargs['x']
                        units = '1'+s[s.find("(")+1:s.find(")")]
                        scaledSortedUnits,sortedUnits = sortSINumerically([units],False,True)
                    else:
                        sortedUnits = [1]
                    scaledNewLevelValues = [float(i) / float(sortedUnits[0]) for i in newLevelValues]
                    plottingDf[kwargs['x']] = scaledNewLevelValues
                    if plotType == 'categorical':
                        kwargs['order'] = scaledNewLevelValues
        else:
            pass
        col_wrap_min = colwrap
        #Make sure there are not more than 6 plots in a row (if no row facet variable)
        if 'row' not in facetgridkwargs.keys():
            if 'col' in facetgridkwargs.keys():
                colwrap = min([len(kwargs['col_order']),col_wrap_min])
                kwargs['col_wrap'] = colwrap
        
        if plotAllVar:
            fullTitleString = createFacetPlotName(folderName,dataType,plotType,subPlotType,legendParameterToLevelNameDict,subsettedDfTitle,levelsPlottedIndividually,useModifiedDf,plotOptions)
            if 'temporaryFirstPlot.png' in os.listdir('plots'):
                os.remove('plots'+dirSep+'temporaryFirstPlot.png')
        else:
            fullTitleString = 'temporaryFirstPlot'
        if len(subsettedDf.index) > 0:
            plotSubsettedFigure(subsettedDf,plottingDf,kwargs,facetgridkwargs,plotSpecificDict,plotType,subPlotType,dataType,fullTitleString,plotOptions,subsettedDfTitle,addDistributionPoints,alternateTitle,subfolderName,titleAdjust,cmap)
        if not plotAllVar:
            break
    sns.set_context('notebook')

def reorderKwargs(oldKwargs,kwargs):
    reorderedKwargs = {}
    for key in kwargs:
        if key in oldKwargs:
            reorderedList = []
            if isinstance(oldKwargs[key],list) or isinstance(oldKwargs[key],tuple):
                for val in oldKwargs[key]:
                    for val2 in kwargs[key]:
                        if isinstance(val,str):
                            if val == val2.strip():
                                reorderedList.append(val2)
                                break
                        else:
                            print('val not in val2')
                if len(reorderedList) == 0:
                    reorderedKwargs[key] = kwargs[key]
                else:
                    reorderedKwargs[key] = reorderedList
            else:
                reorderedKwargs[key] = kwargs[key]
        else:
            reorderedKwargs[key] = kwargs[key]
    return reorderedKwargs

#Will not be needed when seaborn 0.9.1 releases
def sanitizeSameValueLevels(plottingDf,kwargs):
    oldKwargs = kwargs.copy()
    unsanitizedPlottingDf = plottingDf.copy()
    #First find levels that have values that are the same as other levels:
    #Get numeric axes levels
    numericAxes = []
    for axis in ['x','y','z']:
        if axis in kwargs.keys():
            numericAxes.append(kwargs[axis])
    #Exclude levels used for numeric axes (x,y,color)
    nonNumericNameList = []
    for levelName in plottingDf.columns:
        if levelName not in numericAxes:
            nonNumericNameList.append(levelName)
    #Get all possible pairs of nonnumericlevels
    possiblePairs = list(itertools.combinations(nonNumericNameList,2))
    overlappingPairs = []
    for possiblePair in possiblePairs:
        uniqueLevelValues1 = pd.unique(plottingDf[possiblePair[0]])
        uniqueLevelValues2 = pd.unique(plottingDf[possiblePair[1]])
        #If overlap, add both levelnames to list
        if bool(set(uniqueLevelValues1) & set(uniqueLevelValues2)):
            overlappingPairs.append(possiblePair)
    #Add dummy spaces to make level values different while not affecting the legend
    #NEED TO THINK ABOUT HOW THIS AFFECTS NUMERIC VARIABLES
    #NEED TO THINK ABOUT HOW TO DEAL WITH VARIABLES I NEED TO SORT NUMERICALLY; MOST LIKELY BY DETERMINING ORDER BEFORE SANITIZING AND ADDING SPACES TO ORDER
    for overlappingPair in overlappingPairs:
        newVals = []
        for value in plottingDf[overlappingPair[1]]:
            newval = str(value) + ' '
            newVals.append(newval)
        plottingDf[overlappingPair[1]] = newVals
        for kwarg,kwargParent in zip(['row_order','col_order','hue_order','size_order'],['row','col','hue','size']):
            if kwarg in kwargs.keys():
                if kwargs[kwargParent] == overlappingPair[1]:
                    kwargs[kwarg] = list(pd.unique(plottingDf[overlappingPair[1]]))
    kwargs = reorderKwargs(oldKwargs,kwargs)
    return plottingDf,kwargs

def plotSubsettedFigure(subsettedDf,plottingDf,kwargs,facetgridkwargs,plotSpecificKwargs,plotType,subPlotType,dataType,fullTitleString,plotOptions,subsettedDfTitle,addDistributionPoints,alternateTitle,subfolderName,titleAdjust,cmap):

    titleBool = True
    secondPathBool = False
    
    #Add in sharex/sharey options
    if plotType != '1d':
        facetKwargs = {'sharex':plotOptions['X']['share'],'sharey':plotOptions['Y']['share']}
    else:
        facetKwargs = {'sharex':False,'sharey':plotOptions['Y']['share']}
    
    if alternateTitle !=  '':
        fullTitleString = alternateTitle 

    auxillaryKwargs = {}
    auxillaryKwargs['plotType'] = plotType
    auxillaryKwargs['subPlotType'] = subPlotType
    auxillaryKwargs['facetgridkwargs'] = facetgridkwargs
    auxillaryKwargs['plotspecifickwargs'] = plotSpecificKwargs
    if cmap != '':
        if cmap == 'glasbey':
            import colorcet as cc
            cmap = cc.glasbey[:len(pd.unique(plottingDf[kwargs['hue']]))]
        cmapKwarg = {'palette':cmap}
    else:
        cmapKwarg = {}
    auxillaryKwargs['cmap'] = cmapKwarg
    
    #if hei != 5 and asp != 1:
    if plotType == '1d':
        plotOptions['Y']['figureDimensions'] = {'height':hei,'aspect':asp}
    else:
        plotOptions['X']['figureDimensions'] = {'height':hei,'aspect':asp}
    #else:
    #    plotOptions['X']['figureDimensions'] = {}
    #Use appropriate facet plot
    #1D plots: (Histograms and KDEs); use facetgrid with appropriate axis level seaborn function
    if plotType == '1d':
        auxillaryKwargs['dataType'] = dataType
        facetPlotType = fp1D
        #Fix duck typing issue with replots: https://github.com/mwaskom/seaborn/issues/1653
        if 'hue' in kwargs and isinstance(plottingDf[kwargs['hue']][0],str):
            if plottingDf[kwargs['hue']][0].isnumeric():
                for i,x in enumerate(kwargs['hue_order']):
                    kwargs['hue_order'][i] = "$%s$" % x
                plottingDf[kwargs['hue']] = ["$%s$" % x for x in plottingDf[kwargs['hue']]]
    #1.5D/categorical plots (bar/point/box etc.); use catplot figure level seaborn function
    elif plotType == 'categorical':
        auxillaryKwargs['addDistributionPoints'] = addDistributionPoints
        facetPlotType = fpCategorical
    #2D plots (line and scatter); use relplot figure level seaborn function
    elif plotType == '2d':
        facetPlotType = fp2D
    #3D plots (heatmaps and 3d scatter/lineplots); use facet grid with appropriate axis level seaborn function
    elif plotType == '3d':
        facetPlotType = fp3D
    fg = facetPlotType.plot(plottingDf,subsettedDf,kwargs,facetKwargs,auxillaryKwargs,plotOptions)
    #SupTitle
    if titleVar == 'yes':
        #Make room for suptitle
        if titleAdjust == '':
            if 'row' in kwargs:
                if len(kwargs['row_order']) <=2:
                    plt.subplots_adjust(top=0.9)
                else:
                    plt.subplots_adjust(top=0.9+len(kwargs['row_order'])*0.005)
            else:
                plt.subplots_adjust(top=0.8)
        else:
            plt.subplots_adjust(top=float(titleAdjust))
        if subsettedDfTitle != 'All':
            subsettedDfTitle = list(map(str,subsettedDfTitle))
        else:
            subsettedDfTitle = [subsettedDfTitle]
        #Do not include placeholder celltypes in suptitle
        if 'NotApplicable' in subsettedDfTitle:
            subsettedDfTitle.remove('NotApplicable')
        plt.suptitle('-'.join(subsettedDfTitle),fontsize = 'x-large',fontweight='bold')
    
    #Legend
    if legendVar == 'no':
        fg._legend.remove()

    #Save figure
    if subfolderName != '':
        if subfolderName not in os.listdir(plotFolderName):
            os.mkdir(plotFolderName+dirSep+subfolderName)
        fg.fig.savefig(plotFolderName+dirSep+subfolderName+dirSep+fullTitleString+'.png',bbox_inches='tight')
    else:
        fg.fig.savefig(plotFolderName+dirSep+fullTitleString+'.png',bbox_inches='tight')
    if secondPathBool:
        fg.fig.savefig('..'+dirSep+'..'+dirSep+'outputFigures'+dirSep+fullTitleString+'.png',bbox_inches='tight')
    print(fullTitleString+' plot saved')
    plt.clf()



#!/usr/bin/env python3
import os,json,math
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from itertools import groupby
from matplotlib.colors import LogNorm,SymLogNorm
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'
from ..dataprocessing.miscFunctions import reindexDataFrame

dividerLength = 0.16

def add_vline(ax, xpos, ypos,length):
    line = plt.Line2D([xpos, xpos], [ypos, ypos+length],transform=ax.transAxes, color='black')
    line.set_clip_on(False)
    ax.add_line(line)

def add_hline(ax, xpos, ypos,length):
    line = plt.Line2D([xpos, xpos+length], [ypos, ypos],transform=ax.transAxes, color='black')
    line.set_clip_on(False)
    ax.add_line(line)

def draw_borders(ax,df):
    add_hline(ax,0,0,1)
    add_hline(ax,0,1,1)
    add_vline(ax,1,0,1)
    add_vline(ax,0,0,1)

def label_len(my_index,level):
    labels = my_index.get_level_values(level)
    return [(k, sum(1 for i in g)) for k,g in groupby(labels)]

def label_index(ax, df):
    xpos = -1*dividerLength
    scale = 1./df.index.size
    for level in range(len(list(df.index.names)))[::-1]:
        pos = len(df.index)
        for label, rpos in label_len(df.index,level):
            lypos = (pos - .6 * rpos)*scale
            if (label not in list(list(df.index.names)[-1])):
                t = ax.text(xpos+(dividerLength/2), lypos, label, va='center',ha='center', transform=ax.transAxes,rotation=0)
            else:
                t = ax.text(-1*(dividerLength/2), lypos, label, va='center',ha='center', transform=ax.transAxes,rotation=0)
            add_hline(ax, xpos, pos*scale,dividerLength)
            pos -= rpos
        add_hline(ax, xpos, pos*scale,dividerLength)
        xpos -= dividerLength

def label_columns(ax,df):
    ypos = -1./df.index.size
    scale = 1./len(df.columns.values)
    xpos=0
    for timepoint in df.columns.values:
        add_vline(ax,xpos,ypos*0.8,ypos*-1*0.8)
        if str(timepoint).replace('.','',1).isdigit():
            if int(timepoint) % 1 == 0:
                timepoint = int(timepoint)
        ax.text(xpos+scale/2, ypos/2, str(timepoint), ha='center', va='center',transform=ax.transAxes)
        xpos+=scale
    add_vline(ax,xpos,ypos*0.8,ypos*-1*0.8)
    ax.text(0.5, ypos*1.4, df.columns.name, ha='center', va='center',transform=ax.transAxes)

def label_headers(ax,df):
    scale = len(df.index.names)
    xpos = -1*dividerLength*scale
    lineheight = 1+(1./df.index.size)
    for name in df.index.names:
        ax.text(xpos+(dividerLength/2),1+(lineheight-1)/2,name,ha='center',va='center',transform=ax.transAxes,size='x-small')
        xpos+=dividerLength

def returnHeatmapAspectRatios(data,kwargs):
    #16x16 heatmap
    basedim = 16
    scale=0.4
    hbase = 6
    abase = 1.25
    if 'row' in kwargs:
        data = data.xs(kwargs['row_order'][0],level=kwargs['row'])
    if 'col' in kwargs:
        data = data.xs(kwargs['col_order'][0],level=kwargs['col'])
    heatmapdf = data.pivot_table(index=kwargs['y'],columns=kwargs['x'], values=kwargs['z'])
    hstart = max(hbase+scale*hbase*(heatmapdf.index.size-basedim)/basedim,hbase)
    astart = max(abase+scale*abase*(heatmapdf.columns.size-basedim)/basedim,abase)
    if 'row_order' in kwargs:
        h = hstart*len(kwargs['row_order'])*scale
    else:
        h = hstart
    if 'col_order' in kwargs:
        a = astart*len(kwargs['col_order'])*scale
    else:
        a = astart
    return a,h

def reorderDfByExperimentParameters(df,experimentParameters):
    return df

def draw_faceted_heatmap(data,indexingdf,xaxis,yaxis,zaxis,lognorm,cbarticks,logarithmic,symlog,symlognorm,linthresh,**kwargs):
    unsortedPivotedData = data.pivot_table(index=yaxis,columns=xaxis, values=zaxis)
    indexdf = indexingdf.groupby(level=yaxis,sort=False).first()
    data = reindexDataFrame(unsortedPivotedData,indexdf,False)
    if not isinstance(xaxis,list):
        originalColumnOrder = list(pd.unique(indexingdf.index.get_level_values(xaxis)))
        if str(originalColumnOrder[0]).isnumeric():
            originalColumnOrder.sort(key=float)
        data = data[originalColumnOrder]
    if not isinstance(yaxis,list):
        originalRowOrder = list(pd.unique(indexingdf.index.get_level_values(yaxis)))
        if str(originalRowOrder[0]).isnumeric():
            originalRowOrder.sort(key=float)
        data = data.reindex(originalRowOrder)
    plt.axis('off')
    experimentParametersBool = False
    for fn in os.listdir('misc'):
        if 'experimentParameters' in fn:
            experimentParametersBool = True
            experimentParameters = json.load(open('misc'+dirSep+fn,'r')) 
    if experimentParametersBool:
        data = reorderDfByExperimentParameters(data,experimentParameters)
    
    data.columns.name = xaxis
    if logarithmic:
        g = sns.heatmap(data, norm=lognorm,**kwargs,cbar=True,cbar_kws={"ticks": cbarticks,'label':zaxis})
    elif symlog:
        linthresh = int(linthresh) 
        maxlog=int(np.ceil(np.log10(data.values.max())))
        minlog=int(np.ceil(np.log10(-1*data.values.min())))
        tick_locations=([-(10**x) for x in range(-linthresh, minlog+1, 1)][::-1]+[0.0]+[(10**x) for x in range(-linthresh,maxlog+1, 1)] )
        #generate logarithmic ticks
        g = sns.heatmap(data, norm=symlognorm,cbar_kws={'label':zaxis,'ticks':tick_locations,'format':ticker.LogFormatterMathtext()},**kwargs)
    else:
        g = sns.heatmap(data, **kwargs,cbar=True,cbar_kws={'label':zaxis})

    #Add hiearchical level names and borders to heatmap
    ax1 = plt.gca()
    label_index(ax1,data)
    draw_borders(g,data)
    label_columns(ax1,data)
     
    label_headers(ax1,data)

def plot(plottingDf,subsettedDf,kwargs,facetKwargs,auxillaryKwargs,plotOptions):
    if auxillaryKwargs['subPlotType'] == 'heatmap':
        a,h = returnHeatmapAspectRatios(subsettedDf,kwargs)
        if plotOptions['Colorbar']['axisScaling'] == 'Logarithmic':
            minVal = min(plottingDf[kwargs['z']])
            plottingDf[plottingDf[kwargs['z']] == np.nan] = minVal
            if minVal <= 0:
                plottingDf[kwargs['z']] = plottingDf[kwargs['z']]+abs(minVal)+1
        fg = sns.FacetGrid(plottingDf,height=h,aspect=a,gridspec_kws={"wspace":0.4},**auxillaryKwargs['facetgridkwargs'])
        if plotOptions['Colorbar']['axisScaling'] == 'Logarithmic':
            logbool = True
            symlogbool = False
            #Move nans to minVal to allow for appropriate log scaling
            minVal = min(subsettedDf[kwargs['z']])
            subsettedDf[np.isnan(subsettedDf)]=minVal
            if minVal <= 0:
                subsettedDf[kwargs['z']] = subsettedDf[kwargs['z']]+abs(minVal)+1
            log_norm = LogNorm(vmin=subsettedDf[kwargs['z']].min(), vmax=subsettedDf[kwargs['z']].max())
            sym_log_norm = ''
            cbar_ticks = [math.pow(10, i) for i in range(math.floor(math.log10(subsettedDf[kwargs['z']].min())), 1+math.ceil(math.log10(subsettedDf[kwargs['z']].max())))]
            lin_thresh = ''
        elif plotOptions['Colorbar']['axisScaling'] == 'Biexponential':
            logbool = False 
            symlogbool = True
            lin_thresh =plotOptions['Colorbar']['linThreshold']
            sym_log_norm = SymLogNorm(linthresh=float(lin_thresh),linscale=1)
            log_norm = ''
            cbar_ticks= ''
        else:
            logbool = False
            symlogbool = False
            log_norm = ''
            sym_log_norm = ''
            cbar_ticks= ''
            lin_thresh = ''
        
        fg.map_dataframe(draw_faceted_heatmap, indexingdf=subsettedDf,xaxis=kwargs['x'], yaxis=kwargs['y'], zaxis=kwargs['z']
                ,lognorm=log_norm,cbarticks=cbar_ticks,logarithmic=logbool,symlog=symlogbool,symlognorm=sym_log_norm,linthresh=lin_thresh)
        #STILL NEED TO WORK ON PUTTING COLORBARS ONLY AT THE END OF A ROW OF HEATMAPS
        for i in range(len(fg.fig.get_axes())):
            cbarax = fg.fig.get_axes()[i]
            cbarax.set_frame_on(True)
    return fg



#!/usr/bin/env python3 
import json,pickle,os
import numpy as np
import pandas as pd
import tkinter as tk
from plateypus.dataprocessing import initialDataProcessing as idp
from plateypus.dataprocessing import cytokineDataProcessing as cydp
from plateypus.dataprocessing import cellDataProcessing as cdp
from plateypus.dataprocessing import proliferationDataProcessing as pdp
from plateypus.dataprocessing import killingDataProcessing as kdp
from plateypus.dataprocessing import pcrDataProcessing as pcrdp
from plateypus.dataprocessing import singleCellDataProcessing as scdp
#import automatedCBAProcessingGUI as autoCBA
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'

secondPath = '..'+dirSep+'..'+dirSep+'outputData'

concUnit = 1e9
unitPrefixDictionary = {1e12:'pM',1e9:'nM',1e6:'uM',1e3:'mM',1e0:'M'}
concUnitPrefix = unitPrefixDictionary[concUnit]

class DataProcessingStartPage(tk.Frame):
    def __init__(self, master,folderName,expNum,ex_data,bPage):
        tk.Frame.__init__(self, master)
        
        #os.chdir(master.homedirectory+'/'+folderName)
        backPage = bPage

        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l2 = tk.Label(mainWindow, text="""Datatype: """,padx = 20).grid(row=0,column=0,sticky=tk.W)
        l2a = tk.Label(mainWindow, text="Cytokine:",padx = 20).grid(row=1,column=0,sticky=tk.W)
        l2b = tk.Label(mainWindow,text="Cell:",padx = 20).grid(row=2,column=0,sticky=tk.W)
        l2c = tk.Label(mainWindow,text="Proliferation:",padx = 20).grid(row=3,column=0,sticky=tk.W)
        l2e = tk.Label(mainWindow,text="Killing:",padx = 20).grid(row=4,column=0,sticky=tk.W)
        l2d = tk.Label(mainWindow,text="Single Cell:",padx = 20).grid(row=5,column=0,sticky=tk.W)
        l2f = tk.Label(mainWindow,text="PCR:",padx = 20).grid(row=6,column=0,sticky=tk.W)
        
        def dataProcessingMaster(folderName,expNum,dataType,ex_data,useBlankWells,homedirectoryLoc):
            completed = True
            if dataType == 'singlecell' or dataType == 'prolif':
                parameterExtension = 'cell'
            else:
                parameterExtension = dataType
            experimentParameters = json.load(open('misc'+dirSep+'experimentParameters-'+folderName+'-'+parameterExtension+'.json','r'))
            if experimentParameters['format'] == 'plate':
                experimentFormat = 'plate'
                experimentLevelLayoutDict = pickle.load(open('misc'+dirSep+'layoutDict-'+folderName+'-'+parameterExtension+'.pkl','rb'))
            else:
                experimentFormat = 'tube'
                experimentLevelLayoutDict = pickle.load(open('misc'+dirSep+'tubeLayout-'+folderName+'-'+parameterExtension+'.pkl','rb'))
            #experimentLevelLayoutDict = idp.tilePlateLayouts(experimentParameters,levelLayouts)
            if(dataType == 'cyt'):
                calibrationParameters = json.load(open('misc'+dirSep+'CBAcalibrationParameters-'+folderName+'.json','r'))
                numberOfCalibrationSamples = calibrationParameters['Number']
                initialStandardVolume = calibrationParameters['Volume']
                species = calibrationParameters['Species']
                kit = calibrationParameters['Kit']
                sdf = calibrationParameters['SerialDilutionFactor']
                #completeCytokineMWDf = pd.read_pickle(homedirectoryLoc+'misc'+dirSep+'kitDf.pkl').query("Species == @species and Kit == @kit")
                completeCytokineMWDf = pd.read_csv(homedirectoryLoc+'misc'+dirSep+'kitDf.csv').set_index(['Species','Kit']).query("Species == @species and Kit == @kit")
                
                if 'cytRenamingDict-'+folderName+'.json' in os.listdir('misc'):
                    cytRenamingDict = json.load(open('misc'+dirSep+'cytRenamingDict-'+folderName+'.json','r'))
                else:
                    cytRenamingDict = {}
                unparsedCytokines = cydp.calibrateExperiment(folderName,secondPath,concUnit,concUnitPrefix,numberOfCalibrationSamples,initialStandardVolume,sdf,completeCytokineMWDf,cytRenamingDict=cytRenamingDict)
                #Offer user chance to rename cytokines that cannot be parsed from csv
                if len(unparsedCytokines) > 0:
                    completed = False
                    master.switch_frame(cydp.CytokineParsingPage,completeCytokineMWDf,unparsedCytokines,folderName,expNum,ex_data,DataProcessingStartPage,bPage)
                    #master.switch_frame(cydp.CytokineParsingPage,completeCytokineMWDf,unparsedCytokines,folderName,expNum,ex_data,DataProcessingStartPage,bPage)
                    #cytRenamingDict = json.load(open('misc'+dirSep+'cytRenamingDict-'+folderName+'.json','r'))
                    #unparsedCytokines = cydp.calibrateExperiment(folderName,secondPath,concUnit,concUnitPrefix,numberOfCalibrationSamples,initialStandardVolume,sdf,completeCytokineMWDf,cytRenamingDict=cytRenamingDict)
                else:
                    basecytdf = idp.createBaseDataFrame(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
                    cytdf = cydp.createCytokineDataFrame(folderName,basecytdf,concUnitPrefix)
                    idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,cytdf,ex_data) 
            elif(dataType == 'cell'):
                celldf = idp.createBaseDataFrame(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
                idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,celldf,ex_data) 
            elif(dataType == 'prolif'):
                prolifdf = pdp.generateBulkProliferationStatistics(folderName,expNum)
                idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,prolifdf,ex_data) 
            elif(dataType == 'killing'):
                killingdf = kdp.generateBulkKillingStatistics(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
                idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,killingdf,ex_data) 
            elif(dataType == 'pcr'):
                pcrdp.convertPCRinput(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
                pcrdf = idp.createBaseDataFrame(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict).droplevel(['CellType','Marker']).rename({'MFI':'Cp'})
                idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,pcrdf,ex_data) 
                
                #Re-add parsed time values back into experiment parameter file
                #levelLabelDict = experimentParameters['levelLabelDict']
                #levelLabelDict['Time'] = list(killingDf.columns) 
                #experimentParameters['levelLabelDict'] = levelLabelDict
                #with open('misc/experimentParameters-'+folderName+'-'+dataType+'.json', 'w') as fp:
                #    json.dump(experimentParameters, fp)

            elif(dataType == 'singlecell'):
                dataType = 'singlecell'
                if experimentFormat == 'plate':
                    scdp.createPlateSingleCellDataFrame(folderName,experimentParameters,experimentLevelLayoutDict,useBlankWells)
                else:
                    scdf = scdp.createTubeSingleCellDataFrame(folderName,experimentParameters,experimentLevelLayoutDict)
            niceDatatypeNameDict = {'cyt':'Cytokine','cell':'Bulk cell','prolif':'Proliferation','singlecell':'Single cell','killing':'Killing','pcr':'PCR'}
            if completed:
                tk.messagebox.showinfo("Dataframe Created", niceDatatypeNameDict[dataType]+" dataframe created!")
        
        def createDataFrame(dataType):
            dataProcessingMaster(folderName,expNum,dataType,ex_data,v3.get(),master.homedirectory)#,v4.get())

        l3 = tk.Label(mainWindow, text="""Action: """).grid(row=0,column=1,sticky=tk.W)
        
        cytCalibrationParametersButton = tk.Button(mainWindow,text='Enter CBA bead calibration parameters',command=lambda: master.switch_frame(cydp.CalibrationParameterPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        cytCalibrationParametersButton.grid(row=1,column=1,sticky=tk.W)
        #cytCalibrationParametersButton = tk.Button(mainWindow,text='Create CBA gates',command=lambda: master.switch_frame(autoCBA.AutomateCBAStartPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        #cytCalibrationParametersButton.grid(row=1,column=2,sticky=tk.W)
        cytDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('cyt'))
        cytDfButton.grid(row=1,column=2,sticky=tk.W)

        cellDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('cell'))
        cellDfButton.grid(row=2,column=2,sticky=tk.W)
        #cellAbPanelButton = tk.Button(mainWindow,text='Edit antibody panel',command=lambda: master.switch_frame(cdp.MarkerNumberPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        #cellAbPanelButton.grid(row=2,column=2,sticky=tk.W)
        
        prolifGenerationGatesButton = tk.Button(mainWindow,text='Edit generation gates',command=lambda: master.switch_frame(pdp.ProliferationSelectionPage,folderName,expNum,ex_data,DataProcessingStartPage,backPage))
        prolifGenerationGatesButton.grid(row=3,column=1,sticky=tk.W)
        prolifDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('prolif'))
        prolifDfButton.grid(row=3,column=2,sticky=tk.W)
        
        killingDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('killing'))
        killingDfButton.grid(row=4,column=2,sticky=tk.W)

        #completeSingleCellDfButton = tk.Button(mainWindow,text='Create complete dataframes')
        #completeSingleCellDfButton.grid(row=4,column=2,sticky=tk.W)
        singleCellDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('singlecell'))
        singleCellDfButton.grid(row=5,column=2,sticky=tk.W)
        cbWindow = tk.Frame(mainWindow)
        cbWindow.grid(row=5,column=1,sticky=tk.W)
        l3 = tk.Label(cbWindow,text='Use empty wells?').grid(row=0,column=0,sticky=tk.W)
        v3 = tk.BooleanVar(value=False)
        cb = tk.Checkbutton(cbWindow, variable=v3)
        cb.grid(row=0,column=1,sticky=tk.W)
        
        killingDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('pcr'))
        killingDfButton.grid(row=6,column=2,sticky=tk.W)
        
        #rbWindow = tk.Frame(mainWindow)
        #rbWindow.grid(row=4,column=1,sticky=tk.W)
        #l4 = tk.Label(rbWindow,text='Well wrap:').grid(row=0,column=0,sticky=tk.W)
        #v4 = tk.StringVar(value=False)
        #cb = tk.Checkbutton(cbWindow, variable=v3)
        #cb.grid(row=0,column=1,sticky=tk.W)

        for i,button in enumerate([cytDfButton,cellDfButton,prolifDfButton,prolifGenerationGatesButton,singleCellDfButton,killingDfButton]):
            if i == 0:
                requiredFiles = ['CBAcalibrationParameters-'+folderName+'.json']
            elif i == 2:
                requiredFiles = ['singleCellDataFrame-proliferation-'+folderName+'.pkl']
            #Go to separate page where you select how many markers are proliferation, then each marker that serves as proliferation
            elif i == 3:
                requiredFiles = ['initialSingleCellDf-channel-'+folderName+'.h5']
                #requiredFiles = ['logicleProliferationDf.pkl','rawProliferationDf.pkl']
            else:
                requiredFiles = []
            for requiredFile in requiredFiles:
                if requiredFile not in os.listdir('misc')+os.listdir('inputData'+dirSep+'bulkCSVFiles')+os.listdir('outputData'+dirSep+'pickleFiles'):
                    button.config(state=tk.DISABLED)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        #tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)
